#define S_FRAME_SIZE	72

#define S_OLD_R0	68
#define S_PSR		64
#define S_PC		60
#define S_LR		56
#define S_SP		52

#define S_IP		48
#define S_FP		44
#define S_R10		40
#define S_R9		36
#define S_R8		32
#define S_R7		28
#define S_R6		24
#define S_R5		20
#define S_R4		16
#define S_R3		12
#define S_R2		8
#define S_R1		4
#define S_R0		0

#define USR26_MODE      0x00000000
#define FIQ26_MODE      0x00000001
#define IRQ26_MODE      0x00000002
#define SVC26_MODE      0x00000003
#define USR_MODE        0x00000010
#define FIQ_MODE        0x00000011
#define IRQ_MODE        0x00000012
#define SVC_MODE        0x00000013
#define ABT_MODE        0x00000017
#define UND_MODE        0x0000001b
#define SYSTEM_MODE     0x0000001f
#define MODE32_BIT      0x00000010
#define MODE_MASK       0x0000001f
#define PSR_T_BIT       0x00000020
#define PSR_F_BIT       0x00000040
#define PSR_I_BIT       0x00000080
#define PSR_J_BIT       0x01000000
#define PSR_Q_BIT       0x08000000
#define PSR_V_BIT       0x10000000
#define PSR_C_BIT       0x20000000
#define PSR_Z_BIT       0x40000000
#define PSR_N_BIT       0x80000000

#define PSR_f           0xff000000      /* Flags                */
#define PSR_s           0x00ff0000      /* Status               */
#define PSR_x           0x0000ff00      /* Extension            */
#define PSR_c           0x000000ff      /* Control              */

	.macro	irq_set_debug_led
	mov	r2, #0x21c000
	ldr	r3, [r2]
	orr	r3, r3, #0x200000
	str	r3, [r2]
	add	r1, r3, #0x1c
	ldr	r3, [r1]
	bic	r3, r3, #0x200000
	str	r3, [r1]
	.endm


	.macro	irq_clear_debug_led
	mov	r2, #0x21c000
	ldr	r3, [r2]
	orr	r3, r3, #0x200000
	str	r3, [r2]
	add	r1, r3, #0x1c
	ldr	r3, [r1]
	orr	r3, r3, #0x200000
	str	r3, [r1]
	.endm

	.macro get_irq_stack			@ setup IRQ stack
	ldr	sp, __irq_stack_start_word
	ldr	sp, [sp, #0]
	.endm

	.macro	irq_save_user_regs
	sub	sp, sp, #S_FRAME_SIZE
	stmia	sp, {r0 - r12}			@ Calling r0-r12
	add     r8, sp, #S_PC
	stmdb   r8, {sp, lr}^                   @ Calling SP, LR
	str     lr, [r8, #0]                    @ Save calling PC
	mrs     r6, spsr
	str     r6, [r8, #4]                    @ Save CPSR
	str     r0, [r8, #8]                    @ Save OLD_R0
	mov	r0, sp
	.endm

	.macro	irq_restore_user_regs
	ldmia	sp, {r0 - lr}^			@ Calling r0 - lr
	mov	r0, r0
	ldr	lr, [sp, #S_PC]			@ Get PC
	add	sp, sp, #S_FRAME_SIZE
	subs	pc, lr, #4			@ return & move spsr_svc into cpsr
	.endm

	.macro	excpt_set_stack, num
	ldr	sp, __excpt_stack_start_adr
	sub	lr, lr, #4			@ Correction of PC, incorrect for thumb
	ldr	sp, [sp, #4*\num]
	str     lr, [sp, #S_PC-S_FRAME_SIZE]	@ Save calling PC
	mov	lr, #\num
	.endm

	.macro	excpt_save_regs
	sub	sp, sp, #S_FRAME_SIZE
	stmia	sp, {r0 - r7}			@ Calling r0-r7
	add	r7, sp, #S_PC
	mrs	r6, spsr			@ r6 holds saved mode
	str	r6, [r7, #4]                    @ Save CPSR
	str	r0, [r7, #8]                    @ Save OLD_R0
	tst	r6, #0xf
	stmeqdb	r7, {r8 - lr}^			@ Calling r8 - r12, SP, LR for user mode
	beq	55f
	mrs	r5, cpsr			@ r5 holds actual mode
	orr	r6, r6, #MODE32_BIT+PSR_F_BIT+PSR_I_BIT
	bic	r6, r6, #PSR_T_BIT
	msr	cpsr_c, r6
	stmdb	r7, {r8 - lr}			@ Calling r8 - r12, SP, LR for other modes
	msr	cpsr_c, r5
55:	mov	r1, sp
	.endm

	.macro	excpt_restore_regs
	add	r7, sp, #S_PC
	ldr	r6, [r7, #4]                    @ Calling CPSR
	msr	spsr, r6
	tst	r6, #0xf
	ldmeqdb	r7, {r8 - lr}^			@ Calling r8 - r12, SP, LR for user mode
	beq	55f
	mrs	r5, cpsr			@ r5 holds actual mode
	orr	r6, r6, #MODE32_BIT+PSR_F_BIT+PSR_I_BIT
	bic	r6, r6, #PSR_T_BIT
	msr	cpsr_c, r6
	ldmdb	r7, {r8 - lr}			@ Calling r8 - r12, SP, LR for other modes
	msr	cpsr_c, r5
55:	ldmia	sp, {r0 - r7}			@ Calling r0 - r7
	mov	r0, r0
	ldr	lr, [sp, #S_PC]			@ Get PC
	add	sp, sp, #S_FRAME_SIZE
	subs	pc, lr, #0			@ return & move spsr_svc into cpsr
	.endm


.globl __excpt_stack_start
.globl __excpt_vector_target
.globl __excpt_cook_handler

__excpt_stack_start_adr:
	.word	__excpt_stack_start

__excpt_vector_target_adr:
	.word	__excpt_vector_target

__excpt_cook_handler:
	.word	0
	.word	__excpt_cook_undef
	.word	__excpt_cook_swi
	.word	__excpt_cook_pref_abort
	.word	__excpt_cook_data_abort
	.word	0
	.word	__excpt_cook_irq
	.word	__excpt_cook_fiq

	.align	5

__excpt_cook_undef:			@ 1
	excpt_set_stack 1
	b	__excpt_cook_common

__excpt_cook_swi:			@ 2
	excpt_set_stack 2
	b	__excpt_cook_common

__excpt_cook_pref_abort:		@ 3
	excpt_set_stack 3
	b	__excpt_cook_common

__excpt_cook_data_abort:		@ 4
	excpt_set_stack 4
	b	__excpt_cook_common

__excpt_cook_irq:			@ 6
	excpt_set_stack 6
	b	__excpt_cook_common

__excpt_cook_fiq:			@ 7
	excpt_set_stack 7
__excpt_cook_common:
	excpt_save_regs
	mov	r0, lr
	ldr	r3, __excpt_vector_target_adr
	mov	lr, pc
	ldr	pc, [r3, r0, LSL #2]
	excpt_restore_regs

#define CACHE_DLINESIZE 32
#define CACHE_DSEGMENTS 8
#define CACHE_DENTRIES  64

.globl __cpu_coherent_range
@__cpu_coherent_range(start, end)
__cpu_coherent_range:
        bic     r0, r0, #CACHE_DLINESIZE - 1
1:      mcr     p15, 0, r0, c7, c10, 1          @ clean D entry
        mcr     p15, 0, r0, c7, c5, 1           @ invalidate I entry
        add     r0, r0, #CACHE_DLINESIZE
        cmp     r0, r1
        blo     1b
        mcr     p15, 0, r0, c7, c10, 4          @ drain WB
        mov     pc, lr

.global __cpu_enable_icache
__cpu_enable_icache:
        mrc     p15, 0, ip, c1, c0, 0           @ ctrl register
        orr     ip, ip, #0x1000                 @ disable ICACHE
        mcr     p15, 0, ip, c1, c0, 0           @ ctrl register
        mov     pc, lr

.global __cpu_disable_all_cache
__cpu_disable_all_cache:
        mrc     p15, 0, ip, c1, c0, 0           @ ctrl register
        bic     ip, ip, #0x0004                 @ disable ICACHE
        bic     ip, ip, #0x1000                 @ disable DCACHE
        mcr     p15, 0, ip, c1, c0, 0           @ ctrl register

.global __cpu_invalidate_all_cache
__cpu_invalidate_all_cache:
        mov     ip, #0
        mcr     p15, 0, ip, c7, c7, 0           @ invalidate I,D caches
        mcr     p15, 0, ip, c7, c10, 4          @ drain WB
        mcr     p15, 0, ip, c8, c7, 0           @ invalidate I & D TLBs
        mov     pc, lr

.globl __cpu_flush_all_cache
__cpu_flush_all_cache:
        mov     ip, #0
        mov     r1, #(CACHE_DSEGMENTS - 1) << 5 @ 8 segments
1:      orr     r3, r1, #(CACHE_DENTRIES - 1) << 26 @ 64 entries
2:      mcr     p15, 0, r3, c7, c14, 2          @ clean+invalidate D index
        subs    r3, r3, #1 << 26
        bcs     2b                              @ entries 63 to 0
        subs    r1, r1, #1 << 5
        bcs     1b                              @ segments 7 to 0
        mcr     p15, 0, ip, c7, c5, 0           @ invalidate I cache
        mcr     p15, 0, ip, c7, c10, 4          @ drain WB
        mov     pc, lr
