/*
 * linux/arch/arm/mach-pxa/kgdb-serial.c
 *  
 * Provides low level kgdb serial support hooks for PXA2xx boards
 *  
 * Author:	Nicolas Pitre
 * Copyright:	(C) 2002-2005 MontaVista Software Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <cpu_def.h>
#include "gdb-stub.h"
#include <periph/mx1_uart_ll.h>

int kgdb_debug_channel = 1;
int kgdb_debug_baudrate = 19200;

int kgdb_serial_set_parameters(int channel, int baudrate)
{
	if(channel != -1){
		kgdb_debug_channel = channel;
	}
	if(baudrate != -1){
		kgdb_debug_baudrate = baudrate;
	}
	mx1_uartx_setmode(kgdb_debug_channel, kgdb_debug_baudrate, /*mode*/ 0, /*flowc*/ 0);
	return 0;
}

static int kgdb_serial_init(void)
{
	//mx1_uartx_setmode(kgdb_debug_channel, kgdb_debug_baudrate, /*mode*/ 0, /*flowc*/ 0);
	return 1;
}

static void kgdb_serial_putchar(int c)
{
	while(!mx1_uartx_sendready(kgdb_debug_channel));
	mx1_uartx_sendch(kgdb_debug_channel, c);
}

static void kgdb_serial_flush(void)
{
	while(!mx1_uartx_senddone(kgdb_debug_channel));
}

static int kgdb_serial_getchar(void)
{
	while(!mx1_uartx_recready(kgdb_debug_channel));
	return mx1_uartx_recch(kgdb_debug_channel);
}

struct kgdb_io kgdb_io_ops = {
	.init		= kgdb_serial_init,
	.write_char	= kgdb_serial_putchar,
	.flush		= kgdb_serial_flush,
	.read_char	= kgdb_serial_getchar,
};
