#ifndef _FLASHLIB_H
 #define _FLASHLIB_H

#include <types.h>
#include <endian.h>

/* native order can be specified only, 
 * if target supports misaligned operations 
 */
//#define WITH_FLASH_NATIVE_ORDER

//#define WITH_TARGET_BUS32

#ifdef WITH_FLASH_NATIVE_ORDER
  #define FLASH_BYTE_ORDER __BYTE_ORDER
#else /*WITH_FLASH_NATIVE_ORDER*/
  #define FLASH_BYTE_ORDER __BYTE_ORDER
#endif /*WITH_FLASH_NATIVE_ORDER*/


#if 1
  #define FLASH_WR32(adr,val) (*(volatile uint32_t*)(adr)=(val))
  #define FLASH_RD32(adr) (*(volatile uint32_t*)(adr))
  #define FLASH_WR16(adr,val) (*(volatile uint16_t*)(adr)=(val))
  #define FLASH_RD16(adr) (*(volatile uint16_t*)(adr))
  #define FLASH_WR8(adr,val) (*(volatile uint8_t*)(adr)=(val))
  #define FLASH_RD8(adr) (*(volatile uint8_t*)(adr))
#endif

typedef uint32_t flash_addr_t;

typedef struct flash_alg_info {
	/* Sets retid to manufacturer and type ID, returns <0 in case of error */
	int (*check_id)(const struct flash_alg_info *alg, flash_addr_t addr, int retid[2]);
	/* Programs one location of flash (16 bits for width=1 or 2; 8 bits for width8=0) */
	int (*prog)(const struct flash_alg_info *alg, flash_addr_t addr, const void *data, long count);
	/* Erase all sectors overlaped by region from addr of size bytes, size=0 => erase all */
	/* This version is capable only of full erase (size=0) and one sector (size=1) */
	int (*erase)(const struct flash_alg_info *alg, flash_addr_t addr, long size);

	void *context;

	flash_addr_t start;
	flash_addr_t size;
	flash_addr_t sec_size;

} flash_alg_info_t;

int strataflash_check_id_x16(const flash_alg_info_t *alg, flash_addr_t addr, int retid[2]);
int strataflash_prog_x16(const flash_alg_info_t *alg, flash_addr_t addr, const void *data, long count);
int strataflash_erase_x16(const flash_alg_info_t *alg, flash_addr_t addr, long size);

flash_alg_info_t *flash_alg_rellocate(const flash_alg_info_t *orig_alg);

extern flash_alg_info_t *flash_alg_default;
extern flash_alg_info_t *flash_alg_bad;

#endif /*_FLASHLIB_H*/
