/*******************************************************************
  Components for embedded applications builded for
  laboratory and medical instruments firmware  
 
  char_que.h - character queues
 
  Copyright (C) 2001-2003 by Pavel Pisa pisa@cmp.felk.cvut.cz
            (C) 2001-2003 by PiKRON Ltd. http://www.pikron.com

 *******************************************************************/

#include <types.h>
#include <system_def.h>

typedef volatile struct{
  uint8_t *buf_beg;
  uint8_t *buf_end;
  uint8_t *ip;
  uint8_t *op;
} char_que_t;

/* put character c into queue, if full return -1 */
static inline
int char_que_put(char_que_t *q, int c)
{
  uint8_t *p;
  p=q->ip;
  *(p++)=c;
  if(p==q->buf_end) p=q->buf_beg;
  if(p==q->op) return -1;
  q->ip=p;
  return c;
}

/* get character from queue, if empty return -1 */
static inline
int char_que_get(char_que_t *q)
{
  uint8_t *p;
  int  c;
  p=q->op;
  if(p==q->ip) return -1;
  c=*(p++);
  if(p==q->buf_end) p=q->buf_beg;
  q->op=p;
  return c;
}

static inline
int char_que_freecnt(char_que_t *q)
{
  uint8_t *ip=q->ip,*op=q->op;
  return op-ip-1+(op<=ip?q->buf_end-q->buf_beg:0);
}

static inline
int char_que_incnt(char_que_t *q)
{
  uint8_t *ip=q->ip,*op=q->op;
  return ip-op+(ip<op?q->buf_end-q->buf_beg:0);
}
