/*
 *
 * Copyright (C) 2002 Motorola Semiconductors HK Ltd
 *
 */

#ifndef INC_MX1_REG
#define INC_MX1_REG

#include "types.h"

#ifndef MX1_INT_REG_BASE
/*#define MX1_INT_REG_BASE 0xF0000000*/
#define MX1_INT_REG_BASE 0x00000000
#endif

#define __PORT8  (volatile uint8_t  * const)
#define __PORT16 (volatile uint16_t * const)
#define __PORT32 (volatile uint32_t * const)

#ifndef __val2mfld
#define __val2mfld(mask,val) (((mask)&~((mask)<<1))*(val)&(mask))
#endif
#ifndef __mfld2val
#define __mfld2val(mask,val) (((val)&(mask))/((mask)&~((mask)<<1)))
#endif

#ifndef __memory_barrier
#define __memory_barrier() \
__asm__ __volatile__("": : : "memory")
#endif

// ;---------------------------------------;
// ; AIPI2                                 ;
// ; $0021_0000 to $0021_0FFF              ;
// ;---------------------------------------;

#define _reg_PTA_DDIR	(*(__PORT32(MX1_PORTA_BASE+0x00)))


/* AIPI1 control registers */
#define MX1_AIPI1_BASE	(MX1_INT_REG_BASE+0x00200000)             
#define _reg_AIPI1_PSR0	(*(__PORT32(MX1_AIPI1_BASE)))		/* Peripheral Size Reg 0 */
#define _reg_AIPI1_PSR1	(*(__PORT32(MX1_AIPI1_BASE+0x04)))	/* Peripheral Size Reg 1 */
#define _reg_AIPI1_PAR	(*(__PORT32(MX1_AIPI1_BASE+0x08)))	/* Peripheral Access Reg */
#define _reg_AIPI1_PCR	(*(__PORT32(MX1_AIPI1_BASE+0x0C)))	/* Peripheral Control Register */
#define _reg_AIPI1_TSR	(*(__PORT32(MX1_AIPI1_BASE+0x10)))	/* Time-Out Status Register */

/* AIPI2 control registers */
#define MX1_AIPI2_BASE	(MX1_INT_REG_BASE+0x00210000)             
#define _reg_AIPI2_PSR0	(*(__PORT32(MX1_AIPI2_BASE)))		/* Peripheral Size Reg 0 */
#define _reg_AIPI2_PSR1	(*(__PORT32(MX1_AIPI2_BASE+0x04)))	/* Peripheral Size Reg 1 */
#define _reg_AIPI2_PAR	(*(__PORT32(MX1_AIPI2_BASE+0x08)))	/* Peripheral Access Reg */
#define _reg_AIPI2_PCR	(*(__PORT32(MX1_AIPI2_BASE+0x0C)))	/* Peripheral Control Register */
#define _reg_AIPI2_TSR	(*(__PORT32(MX1_AIPI2_BASE+0x10)))	/* Time-Out Status Register */

/* DMA related registers */

#define MX1_DMA_REQ_UART3_T        2
#define MX1_DMA_REQ_UART3_R        3
#define MX1_DMA_REQ_SSI2_T         4
#define MX1_DMA_REQ_SSI2_R         5
#define MX1_DMA_REQ_CSI_STAT       6
#define MX1_DMA_REQ_CSI_R          7
#define MX1_DMA_REQ_MSHC           8
#define MX1_DMA_REQ_DSPA_DCT_DOUT  9
#define MX1_DMA_REQ_DSPA_DCT_DIN  10
#define MX1_DMA_REQ_DSPA_MAC      11
#define MX1_DMA_REQ_EXT           12
#define MX1_DMA_REQ_SDHC          13
#define MX1_DMA_REQ_SPI1_R        14
#define MX1_DMA_REQ_SPI1_T        15
#define MX1_DMA_REQ_SSI_T         16
#define MX1_DMA_REQ_SSI_R         17
#define MX1_DMA_REQ_ASP_DAC       18
#define MX1_DMA_REQ_ASP_ADC       19
#define MX1_DMA_REQ_USP_EP(x)    (20+(x))
#define MX1_DMA_REQ_SPI2_R        26
#define MX1_DMA_REQ_SPI2_T        27
#define MX1_DMA_REQ_UART2_T       28
#define MX1_DMA_REQ_UART2_R       29
#define MX1_DMA_REQ_UART1_T       30
#define MX1_DMA_REQ_UART1_R       31


/* LCDC 0x00205000 to 0x00205FFF */ 
#define MX1_LCDC_BASE	    (MX1_INT_REG_BASE+0x205000)
#define MX1_LCDC_PAL_BASE   (MX1_INT_REG_BASE+0x205800)

#define _reg_LCDC_SSA           (*(__PORT32(MX1_LCDC_BASE+0x00)))
#define _reg_LCDC_SIZE          (*(__PORT32(MX1_LCDC_BASE+0x04)))
#define _reg_LCDC_VPW           (*(__PORT32(MX1_LCDC_BASE+0x08)))
#define _reg_LCDC_CPOS          (*(__PORT32(MX1_LCDC_BASE+0x0C)))
#define _reg_LCDC_LCWHB         (*(__PORT32(MX1_LCDC_BASE+0x10)))
#define _reg_LCDC_LCHCC         (*(__PORT32(MX1_LCDC_BASE+0x14)))
#define _reg_LCDC_PCR           (*(__PORT32(MX1_LCDC_BASE+0x18)))
#define _reg_LCDC_HCR           (*(__PORT32(MX1_LCDC_BASE+0x1C)))
#define _reg_LCDC_VCR           (*(__PORT32(MX1_LCDC_BASE+0x20)))
#define _reg_LCDC_POS           (*(__PORT32(MX1_LCDC_BASE+0x24)))
#define _reg_LCDC_LSCR1         (*(__PORT32(MX1_LCDC_BASE+0x28)))
#define _reg_LCDC_PWMR          (*(__PORT32(MX1_LCDC_BASE+0x2C)))
#define _reg_LCDC_DMACR         (*(__PORT32(MX1_LCDC_BASE+0x30)))
#define _reg_LCDC_RMCR          (*(__PORT32(MX1_LCDC_BASE+0x34)))
#define _reg_LCDC_LCDICR        (*(__PORT32(MX1_LCDC_BASE+0x38)))
#define _reg_LCDC_LCDISR        (*(__PORT32(MX1_LCDC_BASE+0x40)))
#define _reg_LCDC_RAM_0         (*(__PORT32(MX1_LCDC_PAL_BASE+0x00)))
#define _reg_LCDC_RAM_1         (*(__PORT32(MX1_LCDC_PAL_BASE+0x04)))
#define _reg_LCDC_RAM_2         (*(__PORT32(MX1_LCDC_PAL_BASE+0x08)))
#define _reg_LCDC_RAM_3         (*(__PORT32(MX1_LCDC_PAL_BASE+0x0C)))
#define _reg_LCDC_RAM_4         (*(__PORT32(MX1_LCDC_PAL_BASE+0x10)))
#define _reg_LCDC_RAM_5         (*(__PORT32(MX1_LCDC_PAL_BASE+0x14)))
#define _reg_LCDC_RAM_6         (*(__PORT32(MX1_LCDC_PAL_BASE+0x18)))
#define _reg_LCDC_RAM_7         (*(__PORT32(MX1_LCDC_PAL_BASE+0x1C)))
#define _reg_LCDC_RAM_8         (*(__PORT32(MX1_LCDC_PAL_BASE+0x20)))
#define _reg_LCDC_RAM_9         (*(__PORT32(MX1_LCDC_PAL_BASE+0x24)))
#define _reg_LCDC_RAM_a         (*(__PORT32(MX1_LCDC_PAL_BASE+0x28)))
#define _reg_LCDC_RAM_b         (*(__PORT32(MX1_LCDC_PAL_BASE+0x2C)))
#define _reg_LCDC_RAM_c         (*(__PORT32(MX1_LCDC_PAL_BASE+0x30)))
#define _reg_LCDC_RAM_d         (*(__PORT32(MX1_LCDC_PAL_BASE+0x34)))
#define _reg_LCDC_RAM_e         (*(__PORT32(MX1_LCDC_PAL_BASE+0x38)))
#define _reg_LCDC_RAM_f         (*(__PORT32(MX1_LCDC_PAL_BASE+0x3c)))

/* CSPI1 and CSPI2 registers */

struct mx1_cspi_regs {
        volatile uint32_t RXDATA;	/*0x00 - SPI Rx Data Register */
        volatile uint32_t TXDATA;	/*0x04 - SPI Tx Data Register */
        volatile uint32_t CONTROL;	/*0x08 - SPI Control Register */
    #define MX1_CSPI_CONTROL_DATARATE_m	0xE000	/* PERCLK2/4,8,..,512 */
    #define MX1_CSPI_CONTROL_DRCTL_m	0x1800	/* 0=no SPI_RDY, 1=falling edge, 2=low level */
    #define MX1_CSPI_CONTROL_MODE_m	0x0400	/* 1=master, 0=slave */
    #define MX1_CSPI_CONTROL_ENABLE_m	0x0200	/* 1=enable SPI */
    #define MX1_CSPI_CONTROL_XCH_m	0x0100	/* 1=start exchange or busy */
    #define MX1_CSPI_CONTROL_SSPOL_m	0x0080	/* 0=SS active low, 1=SS active high */
    #define MX1_CSPI_CONTROL_SSCTL_m	0x0040	/* 0=SS stay low, 1=SS pulses between bursts */
    #define MX1_CSPI_CONTROL_PHA_m	0x0020	/* 0=Phase 0, 1=Phase 1*/
    #define MX1_CSPI_CONTROL_POL_m	0x0010	/* 0=active high, 1=active low */
    #define MX1_CSPI_CONTROL_BITCOUNT_m	0x000F	/* 0xf=16bit,...,0=1bit transfer */
        volatile uint32_t INT;		/*0x0c - SPI Interrupt Control/Status Register */
    #define MX1_CSPI_INT_BOEN_m		0x8000	/* Bit Count Overflow Interrupt Enable */
    #define MX1_CSPI_INT_ROEN_m		0x4000	/* RXFIFO Overflow Interrupt Enable */
    #define MX1_CSPI_INT_RFEN_m		0x2000	/* RXFIFO Full Interrupt Enable */
    #define MX1_CSPI_INT_RHEN_m		0x1000	/* RXFIFO Half Interrupt Enable */
    #define MX1_CSPI_INT_RREN_m		0x0800	/* RXFIFO Data Ready Interrupt Enable */
    #define MX1_CSPI_INT_TFEN_m		0x0400	/* TXFIFO Full Interrupt Enable */
    #define MX1_CSPI_INT_THEN_m		0x0200	/* TXFIFO Half Interrupt Enable */
    #define MX1_CSPI_INT_TEEN_m		0x0100	/* TXFIFO Empty Interrupt Enable */
    #define MX1_CSPI_INT_ENABLE_m	0xFF00
    #define MX1_CSPI_INT_BO_m		0x0080	/* Bit Count Overflow */
    #define MX1_CSPI_INT_RO_m		0x0040	/* RXFIFO Overflow */
    #define MX1_CSPI_INT_RF_m		0x0020	/* RXFIFO Full Status */
    #define MX1_CSPI_INT_RH_m		0x0010	/* RXFIFO Half Status */
    #define MX1_CSPI_INT_RR_m		0x0008	/* RXFIFO Data Ready Status */
    #define MX1_CSPI_INT_TF_m		0x0004	/* TXFIFO Full Status */
    #define MX1_CSPI_INT_TH_m		0x0002	/* TXFIFO Half Status */
    #define MX1_CSPI_INT_TE_m		0x0001	/* TXFIFO Empty Status */
    #define MX1_CSPI_INT_REQUEST_m	0x00FF
        volatile uint32_t TEST;		/*0x10 - SPI Interrupt Control/Status Register */
    #define MX1_CSPI_TEST_LBC_m		0x4000	/* Loop Back Control */
    #define MX1_CSPI_TEST_SSTATUS_m	0x0F00	/* State Machine Status */
    #define MX1_CSPI_TEST_RXCNT_m	0x00F0	/* RXFIFO Counter */
    #define MX1_CSPI_TEST_TXCNT_m	0x000F	/* TXFIFO Counter */
        volatile uint32_t PERIOD;	/*0x14 - SPI DMA Control Register */
    #define MX1_CSPI_PERIOD_CSRC_m	0x8000	/* 0=bit clock, 1=32kHz clock */
    #define MX1_CSPI_PERIOD_WAIT_m	0x7FFF	/* Number of clocks inserted between transactions */
        volatile uint32_t DMA;		/*0x18 - SPI Sample Period Control Register */
    #define MX1_CSPI_DMA_THDEN_m	0x8000	/* Enables TXFIFO Half DMA Request */
    #define MX1_CSPI_DMA_TEDEN_m	0x4000	/* Enables TXFIFO Empty DMA Requests */
    #define MX1_CSPI_DMA_RFDEN_m	0x2000	/* Enables RXFIFO Full DMA Request */
    #define MX1_CSPI_DMA_RHDEN_m	0x1000	/* Enables RXFIFO Half DMA Request */
    #define MX1_CSPI_DMA_THDMA_m	0x0080	/* TXFIFO Half Status */
    #define MX1_CSPI_DMA_TEDMA_m	0x0040	/* TXFIFO Empty Status */
    #define MX1_CSPI_DMA_RFDMA_m	0x0020	/* RXFIFO Full Status */
    #define MX1_CSPI_DMA_RHDMA_m	0x0010	/* RXFIFO Half Status */
        volatile uint32_t RESET;	/*0x1c - SPI 1 Soft Reset Register */
    #define MX1_CSPI_RESET_START_m	0x0001	/* Executes soft reset. */
};

#define MX1_CSPI1_BASE     (MX1_INT_REG_BASE+0x00213000)
#define _reg_CSPI1	(*(volatile struct mx1_cspi_regs *)(MX1_CSPI1_BASE))

#define MX1_CSPI2_BASE     (MX1_INT_REG_BASE+0x00219000)
#define _reg_CSPI2	(*(volatile struct mx1_cspi_regs *)(MX1_CSPI2_BASE))

/* Register base address */
#define MX1_PLL_BASE	(MX1_INT_REG_BASE+0x0021B000)
//
/* Reset Module*/
#define _reg_PLL_CSCR	(*(__PORT32(MX1_PLL_BASE+0x00)))	/* Clock Source Control Register */
#define _reg_PLL_PCDR	(*(__PORT32(MX1_PLL_BASE+0x20)))	/* Peripherial Clock Divider Register */
/* PLL & Clock Controller */
#define _reg_PLL_MCTL0	(*(__PORT32(MX1_PLL_BASE+0x04)))	/* MCU PLL Control Register 0 */
#define _reg_PLL_MCTL1	(*(__PORT32(MX1_PLL_BASE+0x08)))	/* MCU PLL Control Register 1 */
#define _reg_PLL_UPCTL0	(*(__PORT32(MX1_PLL_BASE+0x0C)))	/* USB PLL Control Register 0 */
#define _reg_PLL_UPCTL1	(*(__PORT32(MX1_PLL_BASE+0x10)))	/* USB PLL Control Register 1 */
/* System Control */
#define _reg_PLL_RSR	(*(__PORT32(MX1_PLL_BASE+0x800)))	/* Reset Source Register */
    #define MX1_PLL_RSR_WDR_m		0x00000002   /* Watchdog reset */
    #define MX1_PLL_RSR_EXR_m		0x00000001   /* External reset */
#define _reg_PLL_SIDR	(*(__PORT32(MX1_PLL_BASE+0x804)))	/* Silicon ID Register */
#define _reg_PLL_FMCR	(*(__PORT32(MX1_PLL_BASE+0x808)))	/* Function Muxing Control Register */
#define _reg_PLL_GPCR	(*(__PORT32(MX1_PLL_BASE+0x80C)))	/* Global Peripherial Control Regiser */


struct mx1_gpio_regs {
	volatile uint32_t DDIR;
	volatile uint32_t OCR1;
	volatile uint32_t OCR2;
	volatile uint32_t ICONFA1;
	volatile uint32_t ICONFA2;
	volatile uint32_t ICONFB1;
	volatile uint32_t ICONFB2;
	volatile uint32_t DR;
	volatile uint32_t GIUS;
	volatile uint32_t SSR;
	volatile uint32_t ICR1;
	volatile uint32_t ICR2;
	volatile uint32_t IMR;
	volatile uint32_t ISR;
	volatile uint32_t GPR;
	volatile uint32_t SWR;
	volatile uint32_t PUEN;
};

#define _reg_PTx(x)	(*(volatile struct mx1_gpio_regs *)(MX1_PORTA_BASE+0x100*(x)))

// PORT A registers
#define MX1_PORTA_BASE	(MX1_INT_REG_BASE+0x21C000)
#define _reg_PTA_DDIR	(*(__PORT32(MX1_PORTA_BASE+0x00)))
#define _reg_PTA_OCR1	(*(__PORT32(MX1_PORTA_BASE+0x04)))
#define _reg_PTA_OCR2	(*(__PORT32(MX1_PORTA_BASE+0x08)))
#define _reg_PTA_ICONFA1 (*(__PORT32(MX1_PORTA_BASE+0x0C)))
#define _reg_PTA_ICONFA2 (*(__PORT32(MX1_PORTA_BASE+0x10)))
#define _reg_PTA_ICONFB1 (*(__PORT32(MX1_PORTA_BASE+0x14)))
#define _reg_PTA_ICONFB2 (*(__PORT32(MX1_PORTA_BASE+0x18)))
#define _reg_PTA_DR	(*(__PORT32(MX1_PORTA_BASE+0x1C)))
#define _reg_PTA_GIUS	(*(__PORT32(MX1_PORTA_BASE+0x20)))
#define _reg_PTA_SSR	(*(__PORT32(MX1_PORTA_BASE+0x24)))
#define _reg_PTA_ICR1	(*(__PORT32(MX1_PORTA_BASE+0x28)))
#define _reg_PTA_ICR2	(*(__PORT32(MX1_PORTA_BASE+0x2C)))
#define _reg_PTA_IMR	(*(__PORT32(MX1_PORTA_BASE+0x30)))
#define _reg_PTA_ISR	(*(__PORT32(MX1_PORTA_BASE+0x34)))
#define _reg_PTA_GPR	(*(__PORT32(MX1_PORTA_BASE+0x38)))
#define _reg_PTA_SWR	(*(__PORT32(MX1_PORTA_BASE+0x3C)))
#define _reg_PTA_PUEN	(*(__PORT32(MX1_PORTA_BASE+0x40)))

#define _reg_PTA	(*(volatile struct mx1_gpio_regs *)(MX1_PORTA_BASE))

// PORT B registers
#define MX1_PORTB_BASE	(MX1_INT_REG_BASE+0x21C100)
#define _reg_PTB_DDIR	(*(__PORT32(MX1_PORTB_BASE+0x00)))
#define _reg_PTB_OCR1	(*(__PORT32(MX1_PORTB_BASE+0x04)))
#define _reg_PTB_OCR2	(*(__PORT32(MX1_PORTB_BASE+0x08)))
#define _reg_PTB_ICONFA1 (*(__PORT32(MX1_PORTB_BASE+0x0C)))
#define _reg_PTB_ICONFA2 (*(__PORT32(MX1_PORTB_BASE+0x10)))
#define _reg_PTB_ICONFB1 (*(__PORT32(MX1_PORTB_BASE+0x14)))
#define _reg_PTB_ICONFB2 (*(__PORT32(MX1_PORTB_BASE+0x18)))
#define _reg_PTB_DR	(*(__PORT32(MX1_PORTB_BASE+0x1C)))
#define _reg_PTB_GIUS	(*(__PORT32(MX1_PORTB_BASE+0x20)))
#define _reg_PTB_SSR	(*(__PORT32(MX1_PORTB_BASE+0x24)))
#define _reg_PTB_ICR1	(*(__PORT32(MX1_PORTB_BASE+0x28)))
#define _reg_PTB_ICR2	(*(__PORT32(MX1_PORTB_BASE+0x2C)))
#define _reg_PTB_IMR	(*(__PORT32(MX1_PORTB_BASE+0x30)))
#define _reg_PTB_ISR	(*(__PORT32(MX1_PORTB_BASE+0x34)))
#define _reg_PTB_GPR	(*(__PORT32(MX1_PORTB_BASE+0x38)))
#define _reg_PTB_SWR	(*(__PORT32(MX1_PORTB_BASE+0x3C)))
#define _reg_PTB_PUEN	(*(__PORT32(MX1_PORTB_BASE+0x40)))

#define _reg_PTB	(*(volatile struct mx1_gpio_regs *)(MX1_PORTB_BASE))

// PORT C registers
#define MX1_PORTC_BASE	(MX1_INT_REG_BASE+0x21C200)
#define _reg_PTC_DDIR	(*(__PORT32(MX1_PORTC_BASE+0x00)))
#define _reg_PTC_OCR1	(*(__PORT32(MX1_PORTC_BASE+0x04)))
#define _reg_PTC_OCR2	(*(__PORT32(MX1_PORTC_BASE+0x08)))
#define _reg_PTC_ICONFA1 (*(__PORT32(MX1_PORTC_BASE+0x0C)))
#define _reg_PTC_ICONFA2 (*(__PORT32(MX1_PORTC_BASE+0x10)))
#define _reg_PTC_ICONFB1 (*(__PORT32(MX1_PORTC_BASE+0x14)))
#define _reg_PTC_ICONFB2 (*(__PORT32(MX1_PORTC_BASE+0x18)))
#define _reg_PTC_DR	(*(__PORT32(MX1_PORTC_BASE+0x1C)))
#define _reg_PTC_GIUS	(*(__PORT32(MX1_PORTC_BASE+0x20)))
#define _reg_PTC_SSR	(*(__PORT32(MX1_PORTC_BASE+0x24)))
#define _reg_PTC_ICR1	(*(__PORT32(MX1_PORTC_BASE+0x28)))
#define _reg_PTC_ICR2	(*(__PORT32(MX1_PORTC_BASE+0x2C)))
#define _reg_PTC_IMR	(*(__PORT32(MX1_PORTC_BASE+0x30)))
#define _reg_PTC_ISR	(*(__PORT32(MX1_PORTC_BASE+0x34)))
#define _reg_PTC_GPR	(*(__PORT32(MX1_PORTC_BASE+0x38)))
#define _reg_PTC_SWR	(*(__PORT32(MX1_PORTC_BASE+0x3C)))
#define _reg_PTC_PUEN	(*(__PORT32(MX1_PORTC_BASE+0x40)))

#define _reg_PTC	(*(volatile struct mx1_gpio_regs *)(MX1_PORTC_BASE))

// PORT D registers
#define MX1_PORTD_BASE	(MX1_INT_REG_BASE+0x21C300)
#define _reg_PTD_DDIR	(*(__PORT32(MX1_PORTD_BASE+0x00)))
#define _reg_PTD_OCR1	(*(__PORT32(MX1_PORTD_BASE+0x04)))
#define _reg_PTD_OCR2	(*(__PORT32(MX1_PORTD_BASE+0x08)))
#define _reg_PTD_ICONFA1 (*(__PORT32(MX1_PORTD_BASE+0x0C)))
#define _reg_PTD_ICONFA2 (*(__PORT32(MX1_PORTD_BASE+0x10)))
#define _reg_PTD_ICONFB1 (*(__PORT32(MX1_PORTD_BASE+0x14)))
#define _reg_PTD_ICONFB2 (*(__PORT32(MX1_PORTD_BASE+0x18)))
#define _reg_PTD_DR	(*(__PORT32(MX1_PORTD_BASE+0x1C)))
#define _reg_PTD_GIUS	(*(__PORT32(MX1_PORTD_BASE+0x20)))
#define _reg_PTD_SSR	(*(__PORT32(MX1_PORTD_BASE+0x24)))
#define _reg_PTD_ICR1	(*(__PORT32(MX1_PORTD_BASE+0x28)))
#define _reg_PTD_ICR2	(*(__PORT32(MX1_PORTD_BASE+0x2C)))
#define _reg_PTD_IMR	(*(__PORT32(MX1_PORTD_BASE+0x30)))
#define _reg_PTD_ISR	(*(__PORT32(MX1_PORTD_BASE+0x34)))
#define _reg_PTD_GPR	(*(__PORT32(MX1_PORTD_BASE+0x38)))
#define _reg_PTD_SWR	(*(__PORT32(MX1_PORTD_BASE+0x3C)))
#define _reg_PTD_PUEN	(*(__PORT32(MX1_PORTD_BASE+0x40)))

#define _reg_PTD	(*(volatile struct mx1_gpio_regs *)(MX1_PORTD_BASE))

// EIM - External Interface Module
#define MX1_EIM_BASE    (MX1_INT_REG_BASE+0x00220000)                
#define _reg_EIM_CS0U   (*(__PORT32(MX1_EIM_BASE+0x00)))
#define _reg_EIM_CS0L   (*(__PORT32(MX1_EIM_BASE+0x04)))
#define _reg_EIM_CS1U   (*(__PORT32(MX1_EIM_BASE+0x08)))
#define _reg_EIM_CS1L   (*(__PORT32(MX1_EIM_BASE+0x0C)))
#define _reg_EIM_CS2U   (*(__PORT32(MX1_EIM_BASE+0x10)))
#define _reg_EIM_CS2L   (*(__PORT32(MX1_EIM_BASE+0x14)))
#define _reg_EIM_CS3U   (*(__PORT32(MX1_EIM_BASE+0x18)))
#define _reg_EIM_CS3L   (*(__PORT32(MX1_EIM_BASE+0x1C)))
#define _reg_EIM_CS4U   (*(__PORT32(MX1_EIM_BASE+0x20)))
#define _reg_EIM_CS4L   (*(__PORT32(MX1_EIM_BASE+0x24)))
#define _reg_EIM_CS5U   (*(__PORT32(MX1_EIM_BASE+0x28)))
#define _reg_EIM_CS5L   (*(__PORT32(MX1_EIM_BASE+0x2C)))
    #define MX1_EIM_CSxU_DTACK_m	0x80000000 /*DTACK Select*/
    #define MX1_EIM_CSxU_BCD_m		0x30000000 /*Burst Clock Divisor*/
    #define MX1_EIM_CSxU_BCS_m		0x0F000000 /*Burst Clock Start*/
    #define MX1_EIM_CSxU_PSZ_m		0x00C00000 /*Page Size*/
    #define MX1_EIM_CSxU_PME_m		0x00200000 /*Page Mode Emulation*/
    #define MX1_EIM_CSxU_SYNC_m		0x00100000 /*Synchronous Burst Mode Enable*/
    #define MX1_EIM_CSxU_DOL_m		0x000F0000 /*Data Output Length*/
    #define MX1_EIM_CSxU_CNC_m		0x0000C000 /*Chip Select Negation Clock Cycles*/
    #define MX1_EIM_CSxU_WSC_m		0x00003F00 /*Wait State Control*/
    #define MX1_EIM_CSxU_WWS_m		0x00000070 /*Write Wait State*/
    #define MX1_EIM_CSxU_EDC_m		0x0000000F /*Extra Dead Cycles*/

    #define MX1_EIM_CSxL_OEA_m		0xF0000000 /*OE Assert delay in half cycles*/
    #define MX1_EIM_CSxL_OEN_m		0x0F000000 /*OE Negate before end of access*/
    #define MX1_EIM_CSxL_WEA_m		0x00F00000 /*EB[0:3] byte enable assert in halves*/
    #define MX1_EIM_CSxL_WEN_m		0x000F0000 /*EB[0:3] negate before end of access */
    #define MX1_EIM_CSxL_CSA_m		0x0000F000 /*Chip select assert delay for write*/
    #define MX1_EIM_CSxL_EBC_m		0x00000800 /*Enable Byte Control for write only*/
    #define MX1_EIM_CSxL_DSZ_m		0x00000700 /*Data Port Size*/
     /* 8-bit port:  000 = D[31:24]; 001 = D[23:16]; 010 = D[15:8]; 011 = D[7:0] */
     /* 16-bit port: 100 = D[31:16]; 101 = D[15:0] */
     /* 11x = 32-bit port */
    #define MX1_EIM_CSxL_SP_m		0x00000040 /*Supervisor Protect*/
    #define MX1_EIM_CSxL_WP_m		0x00000010 /*Write Protect*/
    #define MX1_EIM_CSxL_PA_m		0x00000002 /*Pin Assert*/
    #define MX1_EIM_CSxL_CSEN_m		0x00000001 /*Chip Select Enable*/

// ;---------------------------------------;
// ; SRAMC                                 ;
// ; $0022_1000 to $0022_1FFF              ;
// ;---------------------------------------;
#define MX1_SDRAMC_BASE		(MX1_INT_REG_BASE+0x221000)
#define _reg_SDRAMC_SDCTL0	(*(__PORT32(MX1_SDRAMC_BASE+0x00)))
#define _reg_SDRAMC_SDCTL1	(*(__PORT32(MX1_SDRAMC_BASE+0x04)))
#define _reg_SDRAMC_SDRST	(*(__PORT32(MX1_SDRAMC_BASE+0x18)))


// AITC registers
#define MX1_AITC_BASE		(MX1_INT_REG_BASE+0x223000)
#define _reg_AITC_INTCNTL	(*(__PORT32(MX1_AITC_BASE+0x00)))
    #define MX1_AITC_INTCNTL_NIAD_m	0x00100000
    #define MX1_AITC_INTCNTL_FIAD_m	0x00080000
#define _reg_AITC_NIMASK	(*(__PORT32(MX1_AITC_BASE+0x04)))
#define _reg_AITC_INTENNUM	(*(__PORT32(MX1_AITC_BASE+0x08)))
#define _reg_AITC_INTDISNUM	(*(__PORT32(MX1_AITC_BASE+0x0C)))
#define _reg_AITC_INTENABLEH	(*(__PORT32(MX1_AITC_BASE+0x10)))
#define _reg_AITC_INTENABLEL	(*(__PORT32(MX1_AITC_BASE+0x14)))
#define _reg_AITC_INTTYPEH	(*(__PORT32(MX1_AITC_BASE+0x18)))
#define _reg_AITC_INTTYPEL	(*(__PORT32(MX1_AITC_BASE+0x1C)))
#define _reg_AITC_NIPRIORITY(x)	(*(__PORT32(MX1_AITC_BASE+0x20+4*(7-(x)))))
#define _reg_AITC_NIVECSR	(*(__PORT32(MX1_AITC_BASE+0x40)))
#define _reg_AITC_FIVECSR	(*(__PORT32(MX1_AITC_BASE+0x44)))
#define _reg_AITC_INTSRCH	(*(__PORT32(MX1_AITC_BASE+0x48)))
#define _reg_AITC_INTSRCL	(*(__PORT32(MX1_AITC_BASE+0x4C)))
#define _reg_AITC_INTFRCH	(*(__PORT32(MX1_AITC_BASE+0x50)))
#define _reg_AITC_INTFRCL	(*(__PORT32(MX1_AITC_BASE+0x54)))
#define _reg_AITC_NIPNDH	(*(__PORT32(MX1_AITC_BASE+0x58)))
#define _reg_AITC_NIPNDL	(*(__PORT32(MX1_AITC_BASE+0x5C)))
#define _reg_AITC_FIPNDH	(*(__PORT32(MX1_AITC_BASE+0x60)))
#define _reg_AITC_FIPNDL	(*(__PORT32(MX1_AITC_BASE+0x64)))


#define MX1_INT_SOFTINT                 0
#define MX1_CSI_INT                     6
#define MX1_DSPA_MAC_INT                7
#define MX1_DSPA_INT                    8
#define MX1_COMP_INT                    9
#define MX1_MSHC_XINT                   10
#define MX1_GPIO_INT_PORTA              11
#define MX1_GPIO_INT_PORTB              12
#define MX1_GPIO_INT_PORTC              13
#define MX1_LCDC_INT                    14
#define MX1_SIM_INT                     15
#define MX1_SIM_DATA_INT                16
#define MX1_RTC_INT                     17
#define MX1_RTC_SAMINT                  18
#define MX1_UART2_MINT_PFERR            19
#define MX1_UART2_MINT_RTS              20
#define MX1_UART2_MINT_DTR              21
#define MX1_UART2_MINT_UARTC            22
#define MX1_UART2_MINT_TX               23
#define MX1_UART2_MINT_RX               24
#define MX1_UART1_MINT_PFERR            25
#define MX1_UART1_MINT_RTS              26
#define MX1_UART1_MINT_DTR              27
#define MX1_UART1_MINT_UARTC            28
#define MX1_UART1_MINT_TX               29
#define MX1_UART1_MINT_RX               30
#define MX1_VOICE_DAC_INT               31
#define MX1_VOICE_ADC_INT               32
#define MX1_PEN_DATA_INT                33
#define MX1_PWM_INT                     34
#define MX1_SDHC_INT                    35
#define MX1_I2C_INT                     39
#define MX1_CSPI_INT                    41
#define MX1_SSI_TX_INT                  42
#define MX1_SSI_TX_ERR_INT              43
#define MX1_SSI_RX_INT                  44
#define MX1_SSI_RX_ERR_INT              45
#define MX1_TOUCH_INT                   46
#define MX1_USBD_INT0                   47
#define MX1_USBD_INT1                   48
#define MX1_USBD_INT2                   49
#define MX1_USBD_INT3                   50
#define MX1_USBD_INT4                   51
#define MX1_USBD_INT5                   52
#define MX1_USBD_INT6                   53
#define MX1_BTSYS_INT                   55
#define MX1_BTTIM_INT                   56
#define MX1_BTWUI_INT                   57
#define MX1_TIM2_INT                    58
#define MX1_TIM1_INT                    59
#define MX1_DMA_ERR                     60
#define MX1_DMA_INT                     61
#define MX1_GPIO_INT_PORTD              62


// Watchdog timer registers
#define MX1_WDG_BASE		(MX1_INT_REG_BASE+0x201000)
#define _reg_WDG_WCR		(*(__PORT32(MX1_WDG_BASE+0x00)))
    #define MX1_WDG_WCR_WHALT_m		0x8000	/* Watchdog halt */
    #define MX1_WDG_WCR_WT_m		0x7f00	/* Watchdog timeout */
    #define MX1_WDG_WCR_WIE_m		0x0010	/* Watchdog interrupt enable */
    #define MX1_WDG_WCR_TMD_m		0x0008	/* Test mode enable */
    #define MX1_WDG_WCR_SWR_m		0x0004	/* Software reset eneble */
    #define MX1_WDG_WCR_WDEC_m		0x0002	/* Watchdog enable control */
    #define MX1_WDG_WCR_WDE_m		0x0001	/* Watchdog enable */
#define _reg_WDG_WSR		(*(__PORT32(MX1_WDG_BASE+0x04)))
    #define MX1_WDG_WSR_PHA1_m		0x5555	/* Feed watchdog phase 1 */
    #define MX1_WDG_WSR_PHA2_m		0xAAAA	/* Feed watchdog phase 2 */
#define _reg_WDG_WSTR		(*(__PORT32(MX1_WDG_BASE+0x08)))
    #define MX1_WDG_WSTR_TINT_m		0x0100	/* Timeout interrupt */
    #define MX1_WDG_WSTR_TOUT_m		0x0001	/* Timeout occured */

// Timer 1 registers
#define MX1_TIMER1_BASE		(MX1_INT_REG_BASE+0x202000)
#define _reg_TMR_TCTL1		(*(__PORT32(MX1_TIMER1_BASE+0x00)))
#define _reg_TMR_TPRER1		(*(__PORT32(MX1_TIMER1_BASE+0x04)))
#define _reg_TMR_TCMP1		(*(__PORT32(MX1_TIMER1_BASE+0x08)))
#define _reg_TMR_TSTAT1		(*(__PORT32(MX1_TIMER1_BASE+0x14)))

// Real Time Clock registers
#define MX1_RTC_BASE		(MX1_INT_REG_BASE+0x204000)
#define _reg_RTC_RCCTL		(*(__PORT32(MX1_RTC_BASE+0x10)))
#define _reg_RTC_RTCISR		(*(__PORT32(MX1_RTC_BASE+0x14)))
#define _reg_RTC_RTCIENR	(*(__PORT32(MX1_RTC_BASE+0x18)))

#define MX1_RTC_FRR_m		0x0100  // Free Running/Restart (bit 8)
#define MX1_RTC_IRQEN_m		0x0010  // IRQ enable (bit 4)
#define MX1_RTC_CLKSOURCE_m	0x000E  // clock source (bit 3-1)
#define MX1_RTC_TEN_m		0x0001  // timer enable (bit 0)

// UART registers

struct mx1_uartx_regs {
        volatile uint32_t URXD[16];	/*0x00-0x3c UARTx Receiver Register 0-15 */
    #define MX1_UX_URXD_CHARRDY_m	0x8000
    #define MX1_UX_URXD_ERR_m		0x4000
    #define MX1_UX_URXD_OVRRUN_m	0x2000
    #define MX1_UX_URXD_FRMERR_m	0x1000
    #define MX1_UX_URXD_BRK_m		0x0800
    #define MX1_UX_URXD_PRERR_m		0x0400
    #define MX1_UX_URXD_RX_DATA_m	0x00FF
        volatile uint32_t UTXD[16];	/*0x40-0x7c UARTx Transmitter Register 0-15 */
        volatile uint32_t UCR1;		/*0x80 UARTx Control Register 1 */
    #define MX1_UX_UCR1_ADEN_m		0x8000
    #define MX1_UX_UCR1_ADBR_m		0x4000
    #define MX1_UX_UCR1_TRDYEN_m	0x2000
    #define MX1_UX_UCR1_IDEN_m		0x1000
    #define MX1_UX_UCR1_ICD_m		0x0C00
    #define MX1_UX_UCR1_RRDYEN_m	0x0200
    #define MX1_UX_UCR1_RDMAEN_m	0x0100
    #define MX1_UX_UCR1_IREN_m		0x0080
    #define MX1_UX_UCR1_TXMPTYEN_m	0x0040
    #define MX1_UX_UCR1_RTSDEN_m	0x0020
    #define MX1_UX_UCR1_SNDBRK_m	0x0010
    #define MX1_UX_UCR1_TDMAEN_m	0x0008
    #define MX1_UX_UCR1_UARTCLKEN_m	0x0004
    #define MX1_UX_UCR1_DOZE_m		0x0002
    #define MX1_UX_UCR1_UARTEN_m	0x0001
        volatile uint32_t UCR2;		/*0x84 UARTx Control Register 2 */
    #define MX1_UX_UCR2_ESCI_m		0x8000
    #define MX1_UX_UCR2_IRTS_m		0x4000
    #define MX1_UX_UCR2_CTSC_m		0x2000
    #define MX1_UX_UCR2_CTS_m		0x1000
    #define MX1_UX_UCR2_ESCEN_m		0x0800
    #define MX1_UX_UCR2_RTEC_m		0x0600
    #define MX1_UX_UCR2_PREN_m		0x0100
    #define MX1_UX_UCR2_PROE_m		0x0080
    #define MX1_UX_UCR2_STPB_m		0x0040
    #define MX1_UX_UCR2_WS_m		0x0020
    #define MX1_UX_UCR2_RTSEN_m		0x0010
    #define MX1_UX_UCR2_TXEN_m		0x0004
    #define MX1_UX_UCR2_RXEN_m		0x0002
    #define MX1_UX_UCR2_SRST_m		0x0001
        volatile uint32_t UCR3;		/*0x88 UARTx Control Register 3 */
    #define MX1_UX_UCR3_DPEC_m		0xC000	/* UART2 and UART3 only */
    #define MX1_UX_UCR3_DTREN_m		0x2000	/* UART2 and UART3 only */
    #define MX1_UX_UCR3_PARERREN_m	0x1000
    #define MX1_UX_UCR3_FRAERREN_m	0x0800
    #define MX1_UX_UCR3_DSR_m		0x0400 	/* UART2 and UART3 only */
    #define MX1_UX_UCR3_DCD_m		0x0200	/* UART2 and UART3 only */
    #define MX1_UX_UCR3_RI_m		0x0100	/* UART2 and UART3 only */
    #define MX1_UX_UCR3_RXDSEN_m	0x0040
    #define MX1_UX_UCR3_AIRINTEN_m	0x0020
    #define MX1_UX_UCR3_AWAKEN_m	0x0010
    #define MX1_UX_UCR3_REF25_m		0x0008
    #define MX1_UX_UCR3_REF30_m		0x0004
    #define MX1_UX_UCR3_INVT_m		0x0002
    #define MX1_UX_UCR3_BPEN_m		0x0001
        volatile uint32_t UCR4;		/*0x8C UARTx Control Register 4 */
    #define MX1_UX_UCR4_CTSTL_m		0xFC00
    #define MX1_UX_UCR4_INVR_m		0x0200
    #define MX1_UX_UCR4_ENIRI_m		0x0100
    #define MX1_UX_UCR4_WKEN_m		0x0080
    #define MX1_UX_UCR4_REF16_m		0x0040
    #define MX1_UX_UCR4_IRSC_m		0x0020
    #define MX1_UX_UCR4_TCEN_m		0x0008
    #define MX1_UX_UCR4_BKEN_m		0x0004
    #define MX1_UX_UCR4_OREN_m		0x0002
    #define MX1_UX_UCR4_DREN_m		0x0001
        volatile uint32_t UFCR;		/*0x90 UARTx FIFO Control Register */
    #define MX1_UX_UFCR_TXTL_m		0xFC00
    #define MX1_UX_UFCR_RFDIV_m		0x0380
    #define MX1_UX_UFCR_RXTL_m		0x003F
        volatile uint32_t USR1;		/*0x94 UARTx Status Register 1 */
    #define MX1_UX_USR1_PARITYERR_m	0x8000
    #define MX1_UX_USR1_RTSS_m		0x4000
    #define MX1_UX_USR1_TRDY_m		0x2000
    #define MX1_UX_USR1_RTSD_m		0x1000
    #define MX1_UX_USR1_ESCF_m		0x0800
    #define MX1_UX_USR1_FRAMERR_m	0x0400
    #define MX1_UX_USR1_RRDY_m		0x0200
    #define MX1_UX_USR1_RXDS_m		0x0040
    #define MX1_UX_USR1_AIRINT_m	0x0020
    #define MX1_UX_USR1_AWAKE_m		0x0010
        volatile uint32_t USR2;		/*0x98 UARTx Status Register 2 */
    #define MX1_UX_USR2_ADET_m		0x8000
    #define MX1_UX_USR2_TXFE_m		0x4000
    #define MX1_UX_USR2_DTRF_m		0x2000
    #define MX1_UX_USR2_IDLE_m		0x1000
    #define MX1_UX_USR2_IRINT_m		0x0100
    #define MX1_UX_USR2_WAKE_m		0x0080
    #define MX1_UX_USR2_RTSF_m		0x0010
    #define MX1_UX_USR2_TXDC_m		0x0008
    #define MX1_UX_USR2_BRCD_m		0x0004
    #define MX1_UX_USR2_ORE_m		0x0002
    #define MX1_UX_USR2_RDR_m		0x0001
        volatile uint32_t UESC;		/*0x9C UARTx Escape Character Register */
        volatile uint32_t UTIM;		/*0xA0 UARTx Escape Timer Register */
        volatile uint32_t UBIR;		/*0xA4 UARTx BRM Incremental Register */
        volatile uint32_t UBMR;		/*0xA8 UARTx BRM Modulator Register */
        volatile uint32_t UBRC;		/*0xAC UARTx Baud Rate Count Register */
        volatile uint32_t UBIPR[4];	/*0xBO-0xBC UARTx BRM Incremental Preset Register */
        volatile uint32_t UBMPR[4];	/*0xCO-0xCC UARTx BRM Modulator Preset Register */
        volatile uint32_t UTS;		/*0xDO UARTx Test Register 1 */
    #define MX1_UX_UCR4_FRCPERR_m	0x2000
    #define MX1_UX_UCR4_LOOP_m		0x1000
    #define MX1_UX_UCR4_TXEMPTY_m	0x0040
    #define MX1_UX_UCR4_RXEMPTY_m	0x0020
    #define MX1_UX_UCR4_TXFULL_m	0x0010
    #define MX1_UX_UCR4_RXFULL_m	0x0008
    #define MX1_UX_UCR4_SOFTRST_m	0x0001
};

#define MX1_UART1_BASE     (MX1_INT_REG_BASE+0x00206000)
#define _reg_UART1	(*(volatile struct mx1_uartx_regs *)(MX1_UART1_BASE))

#define MX1_UART2_BASE     (MX1_INT_REG_BASE+0x00207000)
#define _reg_UART2	(*(volatile struct mx1_uartx_regs *)(MX1_UART2_BASE))

#define MX1_UART3_BASE     (MX1_INT_REG_BASE+0x0020A000)
#define _reg_UART3	(*(volatile struct mx1_uartx_regs *)(MX1_UART3_BASE))

// PWM
// $0020_8000 to $0020_8FFF

#define MX1_PWM_BASE		(MX1_INT_REG_BASE+0x00208000)
#define _reg_PWM_PWMC		(*(__PORT32(MX1_PWM_BASE+0x00)))    
    #define MX1_PWM_PWMC_HCTR_m		0x00040000  /* Halfword FIFO Data Swapping */
    #define MX1_PWM_PWMC_BCTR_m		0x00020000  /* Byte FIFO Data Swapping */
    #define MX1_PWM_PWMC_SWR_m		0x00010000  /* Software Reset */
    #define MX1_PWM_PWMC_CLKSRC_m	0x00008000  /* Clock Source */
    #define MX1_PWM_PWMC_PRESCALER_m	0x00007F00  /* Prescaler */
    #define MX1_PWM_PWMC_IRQ_m	  	0x00000080  /* Interrupt Request */
    #define MX1_PWM_PWMC_IRQEN_m	0x00000040  /* Interrupt Request Enable */
    #define MX1_PWM_PWMC_FIFOAV_m	0x00000020  /* FIFO Available */
    #define MX1_PWM_PWMC_EN_m	    	0x00000010  /* Enable */
    #define MX1_PWM_PWMC_REPEAT_m	0x0000000c  /* Sample Repeats */
    #define MX1_PWM_PWMC_CLKSEL_m	0x00000003  /* Clock Selection */	
#define _reg_PWM_PWMS		(*(__PORT32(MX1_PWM_BASE+0x04)))    /* Sample */	
#define _reg_PWM_PWMP		(*(__PORT32(MX1_PWM_BASE+0x08)))    /* Period */	
#define _reg_PWM_PWMCNT		(*(__PORT32(MX1_PWM_BASE+0x0C)))    /* Counter */	

// ASP
// $0021_5000 to $0021_5FFF

#define MX1_ASP_BASE		(MX1_INT_REG_BASE+0x00215000)
#define _reg_ASP_		(*(__PORT32(MX1_ASP_BASE+0x00)))    
#define _reg_ASP_ACNTLCR	(*(__PORT32(MX1_ASP_BASE+0x10)))   /* ASP Control Register */
    #define MX1_ASP_ACNTLCR_ASWB_m	0x08000000   /* Auto Mode Switch Bypass */
    #define MX1_ASP_ACNTLCR_ACAL_m	0x04000000   /* Auto Mode Calibration */
    #define MX1_ASP_ACNTLCR_CLKEN_m	0x02000000   /* Clock Enable */
    #define MX1_ASP_ACNTLCR_SWRST_m	0x00800000   /* Software Reset */
    #define MX1_ASP_ACNTLCR_U_SEL_m	0x00200000   /* U-Channel Resistor Selection */
    #define MX1_ASP_ACNTLCR_AZ_SEL_m	0x00100000   /* Auto-Zero Position Selection */
    #define MX1_ASP_ACNTLCR_AZE_m	0x00008000   /* Auto-Zero Enable */
    #define MX1_ASP_ACNTLCR_AUTO_m	0x00004000   /* Auto Sampling */
    #define MX1_ASP_ACNTLCR_MOD_m	0x00003000   /* Mode */
    #define MX1_ASP_ACNTLCR_SW8_m	0x00000800   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW7_m	0x00000400   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW6_m	0x00000200   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW5_m	0x00000100   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW4_m	0x00000080   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW3_m	0x00000040   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW2_m	0x00000020   /* Switch Control */
    #define MX1_ASP_ACNTLCR_SW1_m	0x00000010   /* Switch Control */
    #define MX1_ASP_ACNTLCR_PADE_m	0x00000002   /* Pen A/D Enable */
    #define MX1_ASP_ACNTLCR_BGE_m	0x00000001   /* Voltage Reference Enable */
#define _reg_ASP_PSMPLRG	(*(__PORT32(MX1_ASP_BASE+0x14)))   /* Pen A/D Sample Rate Control Register */
    #define MX1_ASP_PSMPLRG_DMCNT_m	0x00007000   /* Decimation Ratio Count */
    #define MX1_ASP_PSMPLRG_BIT_SELECT_m 0x00000C00  /* Bit Select */
    #define MX1_ASP_PSMPLRG_IDLECNT_m	0x000003f0   /* Idle Count */
    #define MX1_ASP_PSMPLRG_DSCNT_m	0x0000000f   /* Data Setup Count */
#define _reg_ASP_CMPCNTL		(*(__PORT32(MX1_ASP_BASE+0x30)))   /* Compare Control Register */
    #define MX1_ASP_CMPCNTL_INT_m 	0x00080000   /* Interrupt Status */
    #define MX1_ASP_CMPCNTL_CC_m 	0x00040000   /* Compare Control */
    #define MX1_ASP_CMPCNTL_INSEL_m 	0x00030000   /* Input Select */
    #define MX1_ASP_CMPCNTL_COMPAREVALUE_m 0x0000ffff /* Compare Value */
#define _reg_ASP_ICNTLR		(*(__PORT32(MX1_ASP_BASE+0x18)))   /* Interrupt Control Register */
    #define MX1_ASP_ICNTLR_PUIE_m 	0x00000400   /* Pen-up Enable */
    #define MX1_ASP_ICNTLR_POL_m 	0x00000040   /* Pen Interrupt Polarity */
    #define MX1_ASP_ICNTLR_EDGE_m 	0x00000020   /* Edge Enable */
    #define MX1_ASP_ICNTLR_PIRQE_m 	0x00000010   /* Pen Interrupt Enable */
    #define MX1_ASP_ICNTLR_PFFE_m 	0x00000002   /* Pen FIFO Full Interrupt Enable */
    #define MX1_ASP_ICNTLR_PDRE_m 	0x00000001   /* Pen Data Ready Interrupt Enable */
#define _reg_ASP_ISTATR		(*(__PORT32(MX1_ASP_BASE+0x1c)))   /* Interrupt/Error Status Register */
    #define MX1_ASP_ISTATR_PUIS_m 	0x00000400   /* Pen-up Status */
    #define MX1_ASP_ISTATR_POV_m 	0x00000080   /* Pen Sample Data Overflow */
    #define MX1_ASP_ISTATR_PEN_m 	0x00000040   /* Pen Interrupt */
    #define MX1_ASP_ISTATR_PFF_m 	0x00000002   /* Pen Sample FIFO Full */
    #define MX1_ASP_ISTATR_PDR_m 	0x00000001   /* Pen Data Ready */
#define _reg_ASP_PADFIFO	(*(__PORT32(MX1_ASP_BASE+0x00)))   /* Pen Sample FIFO */
#define _reg_ASP_CLKDIV		(*(__PORT32(MX1_ASP_BASE+0x2c)))   /* Clock Divide Register */
    #define MX1_ASP_CLKDIV_PADC_CLK_m 	0x0000001f   /* PADC Clock Divider */
#define _reg_ASP_FIFO_PTR       (*(__PORT32(MX1_ASP_BASE+0x34)))   /* ASP FIFO Pointer Register */
    #define MX1_ASP_FIFO_PTR_READ_m 	0x0000000f
    #define MX1_ASP_FIFO_PTR_WRITE_m 	0x000000f0

// I2C
// $0021_7000 to $0021_7FFF

#define MX1_I2C_BASE		(MX1_INT_REG_BASE+0x00217000)
#define _reg_I2C_IADR		(*(__PORT32(MX1_I2C_BASE+0x00)))
#define _reg_I2C_IFDR		(*(__PORT32(MX1_I2C_BASE+0x04)))
#define _reg_I2C_I2CR		(*(__PORT32(MX1_I2C_BASE+0x08)))
    #define MX1_I2CR_IEN_m		0x00000080  /* I2C interface enable */
    #define MX1_I2CR_IIEN_m		0x00000040  /* I2C interrupt enable */
    #define MX1_I2CR_MSTA_m		0x00000020  /* master mode and STA/STOP control */
    #define MX1_I2CR_MTX_m		0x00000010  /* master/slave 1:Tx/O:Rx */
    #define MX1_I2CR_TXAK_m		0x00000008  /* 0:master/slave ACK Rx; 1: NAK */
    #define MX1_I2CR_RSTA_m		0x00000004  /* repeated start */
#define _reg_I2C_I2SR		(*(__PORT32(MX1_I2C_BASE+0x0C)))
    #define MX1_I2SR_ICF_m		0x00000080  /* byte transferred */
    #define MX1_I2SR_IAAS_m		0x00000040  /* addressed as slave, clear by WR I2CR */
    #define MX1_I2SR_IBB_m		0x00000020  /* bus busy */
    #define MX1_I2SR_IAL_m		0x00000010  /* arbitration lost, cleared only by SW */
    #define MX1_I2SR_SRW_m		0x00000004  /* after IAAS indicate slave Tx request */
    #define MX1_I2SR_IIF_m		0x00000002  /* I2C interrupt flag */
    #define MX1_I2SR_RXAK_m		0x00000001  /* 0:Rx ACKed, 1:Rx NACKed */
#define _reg_I2C_I2DR		(*(__PORT32(MX1_I2C_BASE+0x10)))


// Clock control module registers
#define MX1_CCM_BASE		(MX1_INT_REG_BASE+0x21B000)
#define _reg_CCM_CSCR		(*(__PORT32(MX1_CCM_BASE+0x00)))
#define _reg_CCM_MPCTL0		(*(__PORT32(MX1_CCM_BASE+0x04)))

// USBD registers
#define MX1_USBD_BASE		(MX1_INT_REG_BASE+0x212000)
#define _reg_USBD_FRAME		(*(__PORT32(MX1_USBD_BASE+0x000)))
    #define MX1_USBD_FRAME_MATCH_m	0x07ff0000
    #define MX1_USBD_FRAME_FRAME_m	0x000007ff
#define _reg_USBD_USB_SPEC	(*(__PORT32(MX1_USBD_BASE+0x004)))
    #define MX1_USBD_SPEC_m		0x00000fff
#define _reg_USBD_STAT		(*(__PORT32(MX1_USBD_BASE+0x008)))
    #define MX1_USBD_STAT_RST_m		0x00000100
    #define MX1_USBD_STAT_SUSP_m	0x00000080
    #define MX1_USBD_STAT_CFG_m		0x00000060
    #define MX1_USBD_STAT_INTF_m	0x00000018
    #define MX1_USBD_STAT_ALTSET_m	0x00000007
#define _reg_USBD_CTRL		(*(__PORT32(MX1_USBD_BASE+0x00C)))
    #define MX1_USBD_CTRL_CMD_OVER_m	0x00000040
    #define MX1_USBD_CTRL_CMD_ERROR_m	0x00000020
    #define MX1_USBD_CTRL_USB_SPD_m	0x00000010
    #define MX1_USBD_CTRL_USB_ENA_m	0x00000008
    #define MX1_USBD_CTRL_UDC_RST_m	0x00000004
    #define MX1_USBD_CTRL_AFE_ENA_m	0x00000002
    #define MX1_USBD_CTRL_RESUME_m	0x00000001
#define _reg_USBD_CFGBSY	(*(__PORT32(MX1_USBD_BASE+0x010))) /*DADR*/
    #define MX1_USBD_CFGBSY_CFG_m	0x80000000
    #define MX1_USBD_CFGBSY_BSY_m	0x40000000
    #define MX1_USBD_CFGBSY_DADR_m	0x000001ff
#define _reg_USBD_EPBUF		(*(__PORT32(MX1_USBD_BASE+0x014))) /*DDAT*/
    #define MX1_USBD_EPBUF_DDAT_m	0x000000ff
#define _reg_USBD_INTR_STAT	(*(__PORT32(MX1_USBD_BASE+0x018)))
    #define MX1_USBD_INTR_WAKEUP_m	0x80000000
    #define MX1_USBD_INTR_MSOF_m	0x00000080  /* 1 .. IRQ active */
    #define MX1_USBD_INTR_SOF_m		0x00000040  /* 1 .. confirm and clear */
    #define MX1_USBD_INTR_RESET_STOP_m	0x00000020
    #define MX1_USBD_INTR_RESET_START_m	0x00000010
    #define MX1_USBD_INTR_RES_m		0x00000008
    #define MX1_USBD_INTR_SUSP_m	0x00000004  /* for MASK register */
    #define MX1_USBD_INTR_FRAME_MATCH_m	0x00000002  /* 1 .. mask off IRQ source */
    #define MX1_USBD_INTR_CFG_CHG_m	0x00000001
#define _reg_USBD_INTR_MASK	(*(__PORT32(MX1_USBD_BASE+0x01C)))
#define _reg_USBD_ENABLE	(*(__PORT32(MX1_USBD_BASE+0x024)))
    #define MX1_USBD_ENABLE_RST_m	0x80000000
    #define MX1_USBD_ENABLE_ENAB_m	0x40000000
    #define MX1_USBD_ENABLE_SUSPEND_m	0x30000000
    #define MX1_USBD_ENABLE_ENDIANING_m	0x10000000
    #define MX1_USBD_ENABLE_TSIZE_m	0x00000001

struct mx1_usbd_ep_regs {
	volatile uint32_t STAT;		/*0x00 - Status/Control Register */
    #define MX1_EP_STAT_BYTE_COUNT_m	0x007f0000
    #define MX1_EP_STAT_SIP_m		0x00000100
    #define MX1_EP_STAT_DIR_m		0x00000080  /*1 IN dev to host */
    #define MX1_EP_STAT_MAX_m		0x00000060
    #define MX1_EP_STAT_MAX_8		    0x0000
    #define MX1_EP_STAT_MAX_16		    0x0020
    #define MX1_EP_STAT_MAX_32		    0x0040
    #define MX1_EP_STAT_MAX_64		    0x0060
    #define MX1_EP_STAT_TYP_m		0x00000018
    #define MX1_EP_STAT_TYP_CTRL	    0x0000
    #define MX1_EP_STAT_TYP_ISO		    0x0008
    #define MX1_EP_STAT_TYP_BULK	    0x0010
    #define MX1_EP_STAT_TYP_INTR	    0x0018
    #define MX1_EP_STAT_ZLPS_m		0x00000004
    #define MX1_EP_STAT_FLUSH_m		0x00000002
    #define MX1_EP_STAT_FORCE_STALL_m	0x00000001
        volatile uint32_t INTR_STAT;	/*0x04 - Interrupt Status Register */
    #define MX1_EP_INTR_FULL_m		0x00000100
    #define MX1_EP_INTR_EMPTY_m		0x00000080  /* 1 .. IRQ active */
    #define MX1_EP_INTR_ERROR_m		0x00000040  /* 1 .. confirm and clear */
    #define MX1_EP_INTR_HIGH_m		0x00000020
    #define MX1_EP_INTR_LOW_m		0x00000010
    #define MX1_EP_INTR_MDEVREQ_m	0x00000008
    #define MX1_EP_INTR_EOT_m		0x00000004  /* for MASK register */
    #define MX1_EP_INTR_DEVREQ_m	0x00000002  /* 1 .. mask off IRQ source */
    #define MX1_EP_INTR_EOF_m		0x00000001
	volatile uint32_t INTR_MASK;	/*0x08 - Interrupt Mask Register*/
	volatile uint32_t FDAT;		/*0x0C - FIFO Data Register*/
	volatile uint32_t FSTAT;	/*0x10 - FIFO Status Register*/
    #define MX1_EP_FSTAT_FRAME0_m	0x08000000
    #define MX1_EP_FSTAT_FRAME1_m	0x04000000
    #define MX1_EP_FSTAT_FRAME2_m	0x02000000
    #define MX1_EP_FSTAT_FRAME3_m	0x01000000
    #define MX1_EP_FSTAT_FRAMEx_m	0x0f000000
    #define MX1_EP_FSTAT_ERROR_m	0x00400000
    #define MX1_EP_FSTAT_OF_m		0x00200000
    #define MX1_EP_FSTAT_UF_m		0x00100000
    #define MX1_EP_FSTAT_FR_m		0x00080000
    #define MX1_EP_FSTAT_FULL_m		0x00040000
    #define MX1_EP_FSTAT_ALARM_m	0x00020000
    #define MX1_EP_FSTAT_EMPTY_m	0x00010000
	volatile uint32_t FCTRL;	/*0x14 - FIFO Status Register*/
    #define MX1_EP_FCTRL_WFR_m		0x20000000
    #define MX1_EP_FCTRL_FRAME_m	0x08000000
    #define MX1_EP_FCTRL_GR_m		0x07000000
	volatile uint32_t LRFP;		/*0x18 - Last Read Frame Pointer Register*/
	volatile uint32_t LWFP;		/*0x1C - Last Read Frame Pointer Register*/
	volatile uint32_t FALRM;	/*0x20 - FIFO Alarm Register*/
	volatile uint32_t FRDP;		/*0x24 - FIFO Alarm Register*/
	volatile uint32_t FWRP;		/*0x28 - FIFO Write Pointer Register*/
	volatile uint32_t res;
};

#define _reg_USBD_EP(x)	(((volatile struct mx1_usbd_ep_regs *)(MX1_USBD_BASE+0x030))[x])


#define MX1_USBD_MDEVREQ_m	0x00000008
#define MX1_USBD_EOT_m		0x00000004
#define MX1_USBD_DEVREQ_m	0x00000002
#define MX1_USBD_EOF_m		0x00000001
#define MX1_USBD_WFR_m		0x20000000
#define MX1_USBD_CMD_OVER_m	0x00000040
#define MX1_USBD_SOF_m		0x00000040
#define MX1_USBD_RST_STOP_m	0x00000020
#define MX1_USBD_RST_START_m	0x00000010
#define MX1_USBD_CFG_CHG_m	0x00000001
#define MX1_USBD_ZLPS_m		0x00000004
#define MX1_USBD_FORCE_STALL_m	0x00000001
// PLAM
#define MX1_USBD_USB_DIV_m	0x1C000000
// end PLAM

#endif /*INC_MX1_REG*/
