#include <types.h>
#include <cpu_def.h>
#include <string.h>

#include "linux_setup.h"
#include "start_tag_app.h"


void start_linux_kernel(void *kernel_addr, unsigned arg0, unsigned mach_type, void *tag_list)
{
  sl_prepare4appstart();

  __asm__ __volatile__ (
	"mov r0,%1\n\t"
	"mov r1,%2\n\t"
	"mov r2,%3\n\t"
	"mov pc,%0\n\t"
    ::"r" (kernel_addr), "r" (arg0), "r" (mach_type), "r" (tag_list)
    : "r0", "r1", "r2"
  );
}

void *linux_add_tag(struct tag **tag_head, __u32 size, __u32 tag)
{
   struct tag *t=*tag_head;
   __u32 *p=(__u32 *)t+size;
   while(p>(__u32 *)t) *(--p)=0;
   
   t->hdr.size=size;
   t->hdr.tag=tag;
   *tag_head=tag_next(t);
   return t;
}

void linux_add_string_tag(struct tag **tag_head, __u32 tag, char *str)
{
  struct tag *t;
  int l=strlen(str)+1+sizeof(struct tag_header);
  t=linux_add_tag(tag_head, (l+3)>>2, tag);
  strcpy((void*)&t->u.cmdline,str);
}

extern char __ram_base;
extern char __ram_end;

#define SDRAM_BASE ((unsigned long)&__ram_base)
#define SDRAM_SIZE ((unsigned long)&__ram_end-(unsigned long)&__ram_base)

void start_linux(unsigned long machtype, char *cmdline)
{
  struct tag *tag_list=(struct tag *)(SDRAM_BASE+0x100);
  struct tag *tag_head, *tag_cur;
  void *kernel_addr=(__u8*)SDRAM_BASE+0x8000;

  cli();  
  
  tag_head=tag_list;
  
  tag_cur=linux_add_tag(&tag_head, tag_size(tag_core), ATAG_CORE);
  tag_cur->u.core.flags = 0;
  tag_cur->u.core.pagesize = 0;
  tag_cur->u.core.rootdev = 0;

  tag_cur=linux_add_tag(&tag_head, tag_size(tag_mem32), ATAG_MEM);
  tag_cur->u.mem.size=SDRAM_SIZE;
  tag_cur->u.mem.start=SDRAM_BASE;

  linux_add_string_tag(&tag_head, ATAG_CMDLINE, cmdline);
  
  linux_add_tag(&tag_head, 0, ATAG_NONE);
  
  start_linux_kernel(kernel_addr, 0, machtype, tag_list);
}

int start_user_tag_application(void *app_base, int flags, int (*deb)(const char *f, ...))
{
  /* application starts 256kB after boot block */
  struct tag *tag_list;
  struct tag *tag_cur;
  /*
  app_base=(unsigned char *)((unsigned)(&start_user_application)&~0x3ffff);
  app_base=(unsigned char *)0x10000000;
  app_base+=0x40000;
  */

  if(!flags)
    flags=TAGSTART_FLG_DO_ALL;

  tag_list=(struct tag *)app_base;
 
  for_each_tag(tag_cur,tag_list){

    switch(tag_cur->hdr.tag){

      case ATAG_MEMDATA:
	if(flags&TAGSTART_FLG_DO_MEMDATA){
          if(deb) deb("AL: %08lx -> AL: MEMDATA to %08x\n",(long)tag_cur,tag_cur->u.memdata.dst);
          memcpy((void*)tag_cur->u.memdata.dst,&tag_cur->u.memdata.data,tag_cur->u.memdata.size);
	}
	if(flags&TAGSTART_FLG_CHECK_DATA){
          if(memcmp((void*)tag_cur->u.memdata.dst,&tag_cur->u.memdata.data,tag_cur->u.memdata.size)){
            if(deb) deb("AL: %08lx -> AL: MEMDATA CHECK with %08x FAILED\n",(long)tag_cur,tag_cur->u.memdata.dst);
	    return -1;
	  }
          if(deb) deb("AL: %08lx -> AL: MEMDATA CHECK with %08x OK\n",(long)tag_cur,tag_cur->u.memdata.dst);
	}
        break;

      case ATAG_MEMCPY:
	if(flags&TAGSTART_FLG_DO_MEMCPY){
          if(deb) deb("AL: %08lx -> AL: MEMCPY to %08x\n",(long)tag_cur,tag_cur->u.memcpy.dst);
          memcpy((void*)tag_cur->u.memcpy.dst,(void*)tag_cur->u.memcpy.src,tag_cur->u.memcpy.size);
	}
        break;

      case ATAG_JUMP:
	if(flags&TAGSTART_FLG_DO_JUMP){
          if(deb) deb("AL: %08lx -> AL: JUMP to %08x\n",(long)tag_cur,tag_cur->u.jump.addr);
          sl_prepare4appstart();
          start_linux_kernel((void*)tag_cur->u.jump.addr, tag_cur->u.jump.r0, tag_cur->u.jump.r1, (void*)tag_cur->u.jump.r2);
	}
        break;

      case 0xffffffff:
        if(flags&TAGSTART_FLG_FF_EOF)
	  return 0;
        if(deb) deb("AL: %08lx -> AL: FFFFFFFF tag detected, aborting\n",(long)tag_cur);
        return -1;

      default:
        if(deb) deb("AL: %08lx -> AL: unknow tag %08x\n",(long)tag_cur,tag_cur->hdr.tag);
    }
  }
  return 0;
}
