#include <cpu_def.h>
#include <irq_def.h>
#include <mx1_def.h>
#include <mx1_reg.h>
#include <types.h>
#include <stdarg.h>
#include <char_que.h>
#include <string.h>
#include <periph/mx1_boardctrl.h>

extern char __ram_base;
extern char __ram_end;
extern unsigned char __flashusr_base;

#define SDRAM_BASE ((unsigned long)&__ram_base)
#define SDRAM_SIZE ((unsigned long)&__ram_end-(unsigned long)&__ram_base)

/*
#define SDRAM_BASE 0x08000000
#define SDRAM_SIZE 0x02000000
*/

void sl_udelay(long wait)
{
  int l;
  while(wait--){
    for(l=50;l;l--){
      __asm__ __volatile__ ("":::"memory");
    }
  }
}

void sl_prepare4appstart(void)
{
  int i;

  cli();
  fiq_cli();
  _reg_AITC_INTCNTL=0;
  _reg_AITC_INTENABLEH=0;
  _reg_AITC_INTENABLEL=0;
  _reg_AITC_INTTYPEH=0;
  _reg_AITC_INTTYPEL=0;
  for(i=0;i<7;i++)
    _reg_AITC_NIPRIORITY(i)=0;
  __cpu_disable_all_cache();
}

int mx1_sdram_init(void)
{
  int i;

  //*(volatile long*)SDRAMC_SDRST=0xc0000000;  /* Reset SDRAM controller */
  sl_udelay(200);
  *(volatile long*)SDRAMC_SDCTL0=0x92110334; /* Set Precharge Command */
  sl_udelay(200);
  *(volatile short*)(SDRAM_BASE+0x100000)=0; /* Issue Precharge all Command (assert SDRAM memory A10) */
  *(volatile long*)SDRAMC_SDCTL0=0xA2110334; /* Set AutoRefresh Command */
  for(i=0;i<8;i++){
    *(volatile short*)SDRAM_BASE=0;    /* Issue AutoRefresh Command */
    sl_udelay(20);
  }
  *(volatile long*)SDRAMC_SDCTL0=0xB2110334;  /*Set Mode Register*/
  *(volatile short*)(SDRAM_BASE+0x08CC00)=0; /* Issue Mode Register Command, Burst length = 8, CAS = 3 */
  *(volatile long*)SDRAMC_SDCTL0=0x8211F334;  /* Set to Normal Mode */

  return 0;
}

void mx1_sdram_halt(void)
{
  _reg_SDRAMC_SDRST=0xc0000000;  /* Reset SDRAM controller */
  sl_udelay(20);
  _reg_SDRAMC_SDCTL0=0x01000300;
  sl_udelay(20);
  _reg_SDRAMC_SDRST=0xc0000000;  /* Reset SDRAM controller */
  sl_udelay(20);
}

void mx1_sdram_refresh_off(void)
{
  _reg_SDRAMC_SDCTL0&=~0x0000c000;
}

int mx1_hw_init(void)
{
  int res=0;
  
  /*CLOCK: System PLLCLK = 96MHz*/
  /* SPCTL0  PD=1 MFD=312(0x138) MFI=5 MFN=226(0xE2) */
  *(volatile long*)PLL_UPCTL0=0x053814E2;
  sl_udelay(500);

  /*CLKO_SEL=1 => HCLK output, USB_DIV=1 (/2), SD_CNT=3, SPLL_RESTART=1 
    MPLL_RESTART=1, CLK16_SEL=0, OSC_EN=0, SYSTEM_SEL=0, PRESC=0,
    BCLK_DIV=1, SPEN=1, MPEN=0 */
//  *(volatile long*)PLL_CSCR=0x27600402; 

  /*CLKO_SEL=0 => PERCLK1 output, USB_DIV=1 (/2), SD_CNT=3, SPLL_RESTART=1 
    MPLL_RESTART=1, CLK16_SEL=0, OSC_EN=0, SYSTEM_SEL=0, PRESC=0,
    BCLK_DIV=0, SPEN=1, MPEN=0 */
  *(volatile long*)PLL_CSCR=0x07600002; 
  /* *(long*)PLL_CSCR=0x00600000; SPLL_RESTART, MPLL_RESTART */
  sl_udelay(500);

  /* Pin multiplexing register */
  /* select CSD0 over CS2, CS3 over CSD1 */
  _reg_PLL_FMCR = 0x00000001;

  if(((long)&mx1_hw_init ^ 0x08000000) & ~0x01ffffff) {
    if(mx1_sdram_init()<0)
      res=-1;
  }

  /*FLASH INIT:*/
  //*(volatile long*)EIM_CS0H=0x00000800;    /* CS0U */
  *(volatile long*)EIM_CS0H=0x00004f42;    /* CS0U */
  *(volatile long*)EIM_CS0L=0x00000D01;    /* CS0L */

  /* Erratum 13, Port A pin 17 (multiplexed with DTACK) inadvertently 
   * affects other chip-select functions when DTACK is not used. 
   */
  _reg_PTA_DDIR&=~(1<<17); /* Pin is input */
  _reg_PTA_PUEN|=1<<17;    /* pulled high */
 #if 1
  _reg_PTA_GIUS&=~(1<<17);    /* used as DTACK/IOWAIT */
  _reg_PTA_GPR &=~(1<<17);    /* used as DTACK/IOWAIT */
 #else
  _reg_PTA_GIUS|=1<<17;    /* used for dummy IO */
 #endif

  /* Set PA22 (CS4) and PA23 (CS5) to chipselect function */
  _reg_PTA_DR   |= (1<<22) | (1<<23);    /* Output value high */
  _reg_PTA_DDIR |= (1<<22) | (1<<23);    /* Pin is output */
  _reg_PTA_PUEN &= ~((1<<22) | (1<<23)); /* not-pulled high */
  _reg_PTA_GIUS &= ~((1<<22) | (1<<23)); /* chip select function */
  /*_reg_PTA_GPR |= 1<<23; used from U-Boot, but incorrect probably */


  /*RTC:*/
  *(volatile long*)RTC_RTCCTL=0x00000080;

  /*Chip Selects Initialization:*/
  _reg_EIM_CS4U = MX1_EIM_CSxU_DTACK_m| __val2mfld(MX1_EIM_CSxU_CNC_m,3) |
                  __val2mfld(MX1_EIM_CSxU_WSC_m,8) | __val2mfld(MX1_EIM_CSxU_WWS_m,2) |
		  __val2mfld(MX1_EIM_CSxU_EDC_m,4);
		  
  _reg_EIM_CS4L = __val2mfld(MX1_EIM_CSxL_OEA_m,2) | __val2mfld(MX1_EIM_CSxL_OEN_m,1) |
                  __val2mfld(MX1_EIM_CSxL_WEA_m,2) | __val2mfld(MX1_EIM_CSxL_WEN_m,1) |
		  MX1_EIM_CSxL_EBC_m | __val2mfld(MX1_EIM_CSxL_DSZ_m,4) | MX1_EIM_CSxL_CSEN_m;
		  
/*  _reg_EIM_CS5U = MX1_EIM_CSxU_DTACK_m| __val2mfld(MX1_EIM_CSxU_CNC_m,3) |
 *                __val2mfld(MX1_EIM_CSxU_WSC_m,12) | __val2mfld(MX1_EIM_CSxU_WWS_m,2) |
 *                __val2mfld(MX1_EIM_CSxU_EDC_m,4);
 */
  _reg_EIM_CS5U = (0*MX1_EIM_CSxU_DTACK_m)| __val2mfld(MX1_EIM_CSxU_CNC_m,0) |
                  __val2mfld(MX1_EIM_CSxU_WSC_m,7) | __val2mfld(MX1_EIM_CSxU_WWS_m,0) |
                  __val2mfld(MX1_EIM_CSxU_EDC_m,0);

/*  _reg_EIM_CS5L = __val2mfld(MX1_EIM_CSxL_OEA_m,2) | __val2mfld(MX1_EIM_CSxL_OEN_m,0) |
 *                 __val2mfld(MX1_EIM_CSxL_WEA_m,2) | __val2mfld(MX1_EIM_CSxL_WEN_m,1) |
 *                 MX1_EIM_CSxL_EBC_m | __val2mfld(MX1_EIM_CSxL_DSZ_m,5) |
 *                 MX1_EIM_CSxL_PA_m | MX1_EIM_CSxL_CSEN_m;
 */
  _reg_EIM_CS5L = __val2mfld(MX1_EIM_CSxL_OEA_m,2) | __val2mfld(MX1_EIM_CSxL_OEN_m,2) |
                  __val2mfld(MX1_EIM_CSxL_WEA_m,2) | __val2mfld(MX1_EIM_CSxL_WEN_m,2) |
                  MX1_EIM_CSxL_EBC_m | __val2mfld(MX1_EIM_CSxL_DSZ_m,5) |
                  MX1_EIM_CSxL_PA_m | MX1_EIM_CSxL_CSEN_m;

  /*Enable internal registers access for user applications:*/
  _reg_AIPI1_PAR = 0;
  _reg_AIPI2_PAR = 0;
  
  return res;
}

void mx1_ioports_init(void)
{
  /*
  PC.5  - IRC input A
  PC.6  - IRC input B
  PC.24 - boot select pin
  PC.21 - enable power DM9000 - active low
  PC.19 - power-on reset of DM9000 - active low
  PC.25 - reset of DM9000 - active high
  PC.27 - interrupt from DM9000 - active low
  */

  unsigned mask = (1<<5)|(1<<6)|(1<<19)|(1<<21)|(1<<24)|(1<<25)|(1<<27);

  *(volatile long*)PTC_PUEN  |= (1<<5)|(1<<6)|(1<<24)|(1<<27) ;
  *(volatile long*)PTC_DR    &= ~mask ;
  *(volatile long*)PTC_DR    |= (1<<21) | (1<<25) ;
  *(volatile long*)PTC_DDIR  &= ~mask ;
  *(volatile long*)PTC_DDIR  |= (1<<19)|(1<<21)|(1<<25) ;
  *(volatile long*)PTC_GIUS  =  mask ;
  *(volatile long*)PTC_OCR2  |= (3<<((19-16)*2))|(3<<((21-16)*2))|(3<<((25-16)*2)) ;
  
}

int mx1_get_boot_select(void)
{
  return ((*(volatile long*)PTC_SSR) & (1<<24))?0:1;
}

void mx1_lcd_buff_enable(int val)
{
  if(val) *(volatile long*)PTD_DR &= ~0x200;
  else  *(volatile long*)PTD_DR |= 0x200;
}

static unsigned char mx1_kbd_led_shadow;

void mx1_kbd_led_switch_off(void)
{
  /* warning interferes with KBD read */
  *(volatile long*)PTD_DR &= ~0x7f880000;
  *(volatile long*)PTD_DR |= 0x80000000;
  *(volatile long*)PTD_DR |= 0x00080000;
}

void mx1_kbd_led_shadow_wr(void)
{
  /* warning interferes with KBD read */
  *(volatile long*)PTD_DR &= ~0x7f880000;
  *(volatile long*)PTD_DR |= (long)mx1_kbd_led_shadow<<23;
  *(volatile long*)PTD_DR |= 0x00080000;
}

void mx1_lcd_backlight_set(int val)
{
  if(val<0) val=0;
  else if(val>7) val=7;
  if(val) val=8-val;
  mx1_kbd_led_shadow &= ~0xe0;
  mx1_kbd_led_shadow |= (val<<5);
  
  mx1_kbd_led_shadow_wr();
}

void mx1_lcd_contrast_set(int val)
{
  /* PWMR  SCR=2 CC_EN=1  PWM=0x80 */
  *(volatile long*)LCDC_pwm=0x00000500 | (0xff & val);
}

void mx1_kbd_led_set(int val)
{
  mx1_kbd_led_shadow &= ~0x1f;
  mx1_kbd_led_shadow |= 0x1f & val;

  mx1_kbd_led_shadow_wr();
}

void mx1_kbd_led_modify(int mask_val, int xor_val)
{
  mask_val &= 0x1f;
  xor_val &= 0x1f;
  mx1_kbd_led_shadow &= ~mask_val;
  mx1_kbd_led_shadow ^= xor_val;

  mx1_kbd_led_shadow_wr();
}


int mx1_lcd_gray_pallet_set(int max)
{
  int l;
  for(l=0;l<max;l++){
    ((volatile long*)LCDC_PAL_ADDR)[l]=l;
  }
  return 0;
}

unsigned char *mx1_lcd_base_addr;

int mx1_lcd_init(void)
{
  *(volatile long*)PTD_GIUS=0x7FF80200;
  *(volatile long*)PTD_OCR1=0x000C0000;
  *(volatile long*)PTD_OCR2=0x3FFFC0C0;
  *(volatile long*)PTD_DDIR=0x7F880200;

  mx1_lcd_buff_enable(0);

  mx1_lcd_contrast_set(0x18);
  mx1_lcd_backlight_set(0);

  mx1_lcd_gray_pallet_set(16);
  
  /* CLOCK: */
  *(volatile long*)PLL_PCDR |= 0x00000f0;  /* PCDR  PCLKDIV2=4(/5) => 19.2MHz */
  *(volatile long*)PLL_PCDR &= 0x0000040 | ~0x00000f0;  /*0x000B003B*/

  /*CONTROLER:*/

  *(volatile long*)LCDC_self=0;  /* LCD_EN=0 SELFREF=0 */

  /* *(long*)LCDC_ssa=eSRAM_ADDR_TOP-(320*200/2); */
  mx1_lcd_base_addr=(char*)(eSRAM_ADDR_BOT+0x10000);
  *(volatile long*)LCDC_ssa=(long)mx1_lcd_base_addr;
  
  *(volatile long*)LCDC_xymax = (320<<16) | 240; /* SIZE XMAX=0x140(320) YMAX=0xF0(240) */
  *(volatile long*)LCDC_vpw =    320/8;
  *(volatile long*)LCDC_hcc_xy=  0;              /* CPOS CC=0 (Cursor Off) */

  *(volatile long*)LCDC_con=0x24029803;  /*PCR TFT=0 COL=0 PBSIZ=2(4bit)
  		   BPIX=2(4bpp) PIXPOL=0 FLMPOL=0 LPPOL=0 CLKPOL=0 OEPOL=0 
		   SCLKIDL=0 END_SEL=0(LITTLE) BYTESWAP=1 REVVS=0 ACDSEL=1(HSYNC)
		   ACD=0x18 SCLKSEL=0 SHARP=0 PCD=3(/4)  */
  
  /* *(volatile long*)LCDC_con=0x24049803; */ /*PCR TFT=0 COL=0 PBSIZ=2(4bit)
  		   BPIX=2(4bpp) PIXPOL=0 FLMPOL=0 LPPOL=0 CLKPOL=0 OEPOL=0 
		   SCLKIDL=0 END_SEL=1(BIG) BYTESWAP=0 REVVS=0 ACDSEL=1(HSYNC)
		   ACD=0x18 SCLKSEL=0 SHARP=0 PCD=3(/4)  */

  *(volatile long*)LCDC_hsyn=0x00040000; /* HCR   H_WIDTH=1  H_WAIT1=0  H_WAIT2=0 */
  *(volatile long*)LCDC_vsyn=0x00040001; /* VCR   V_WIDTH=1  V_WAIT1=0  V_WAIT2=1 */
  *(volatile long*)LCDC_pan =0x00000000; /* POS=0 */

  *(volatile long*)LCDC_self=0x2;  /* LCD_EN=1 SELFREF=0 */

  sl_udelay(500);
  
  mx1_lcd_buff_enable(1);

  memset(mx1_lcd_base_addr,0,320/2*240);
  
  return 0;
}

int mx1_uart1_init(void)
{
  /* CLOCK: */
  *(volatile long*)PLL_PCDR |= 0x000000f;  /* PCDR  PCLKDIV1=5(/6) => 16MHz, was 3 => 24MHz */
  *(volatile long*)PLL_PCDR &= 0x0000005 | ~0x000000f;  /*0x000B003B*/
  /* PORT: */
  *(volatile long*)PTC_GIUS&= ~0x00001800 ; /* GIUS_C */
  *(volatile long*)PTC_GPR&= ~0x00001800 ; /* GPR_C */
  /* CONTROL: */
  *(volatile long*)UART1_CR1=0x00000005;
  *(volatile long*)UART1_CR2=0x00004026;
  *(volatile long*)UART1_CR3=0x00000000;
  *(volatile long*)UART1_CR4=0x00008000;
  
  *(volatile long*)UART1_FCR=0x00008201;  /* 0x00008201 */
  /* BAUD: (baud * 16) / uartclk  = (UBIR - 1) / (UBMR - 1) */
  *(volatile long*)UART1_BIR=384-1;  /* 255; */
  *(volatile long*)UART1_BMR=10000-1;
  
  return 0;
}


int mx1_uart2_init(void)
{
  /* CLOCK: */
  *(volatile long*)PLL_PCDR |= 0x000000f;  /* PCDR  PCLKDIV1=5(/6) => 16MHz, was 3 => 24MHz */
  *(volatile long*)PLL_PCDR &= 0x0000005 | ~0x000000f;  /*0x000B003B*/
  /* PORT: */
  *(volatile long*)PTB_GIUS&= ~0xC0000000 ; /* GIUS_B */
  *(volatile long*)PTB_GPR&= ~0xC0000000 ; /* GPR_B */
  /* CONTROL: */
  *(volatile long*)UART2_CR1=0x00000005;
  *(volatile long*)UART2_CR2=0x00004026;
  *(volatile long*)UART2_CR3=0x00000000;
  *(volatile long*)UART2_CR4=0x00008000;
  
  *(volatile long*)UART2_FCR=0x00008201; /* 0x00008201 */
  /* BAUD: */
  *(volatile long*)UART2_BIR=384-1;  /* 255; */
  *(volatile long*)UART2_BMR=10000-1;
  
  return 0;
}

int mx1_hw_mclk_on(void)
{
  /*uint32_t f_mcu=144000000;
  uint32_t f_ref=32768;*/

  *(volatile long*)PLL_CSCR|=0x00000001; /* Enable MCU PLL */

  /* Fdppl=2*Fref*(MFI+MFN/(MFD+1))/(PD+1) */
  /*  PD=<0,15>, MFD=<1,1023>, MFI=<5,15> MFN=<0,1022> */

  /*CLOCK: System PLLCLK = 96MHz*/
  /* SPCTL0  PD=1 MFD=312(0x138) MFI=5 MFN=226(0xE2) */
  {
    uint32_t pd=1, mfd=312, mfi=8, mfn=183;
  
    *(volatile long*)PLL_MCTL0=(pd<<26)|(mfd<<16)|(mfi<<10)|mfn;
    sl_udelay(500);
  }

  *(volatile long*)PLL_CSCR|=0x00200000; /* Restart MCU PLL */

  while(*(volatile long*)PLL_CSCR&0x00200000);
  
  sl_udelay(50000);

  __asm__ __volatile__ (
    "mrc p15,0,r0,c1,c0,0\n\t"
    "orr r0,r0,#0xC0000000\n\t"
    "mcr p15,0,r0,c1,c0,0\n\t"
  );

  return 0;
}


int mx1_power_button_state(void)
{
  return (_reg_PTD_SSR&(1<<POWER_ON_BUTTON_ON_PTD_b))?0:1;
}
