#define NOFLOAT
//#define WITH_USB
#define WITH_USB_SIMPLE
//#define WITH_SLEEP_WAKE
//#define WITH_IRQ_CATCH
//#define WITH_GDB_STUB

#include <cpu_def.h>
#include <irq_def.h>
#include <mx1_def.h>
#include <mx1_reg.h>
#include <types.h>
#include <stdarg.h>
#include <char_que.h>
#include <string.h>

#include <periph/mx1_boardctrl.h>

#include <misc/start_tag_app.h>
#include <misc/linux_start.h>

#ifdef WITH_USB
  #include <usb/usb.h>
  #include <usb_mx1/mx1_usb.h>
#endif /*WITH_USB*/

#ifdef WITH_USB_SIMPLE
  #include <usb_mx1/mx1_usb.h>
  #include <usb_mx1/mx1_usb_ll.h>
#endif /*WITH_USB_SIMPLE*/

extern unsigned char __flashusr_base;

#if 0
/* hack for start of main, should use crt0.o instead */
__asm__ /*__volatile__*/(
	".global _start\n"
	"_start : \n\t"
	"	ldr   sp,2f\n\t"
	"	bl    main\n\t"
	"	b     0x0\n\t"
//	"1:	b     1b\n\t"
	"2:	.long 0x31ff00\n\t"
	);
#endif

/***********************************************************/

#define CHAR_QUE_START ((uint8_t *)0x08000000+0x01000000)
char_que_t rs232_que_out = {
  .buf_beg=CHAR_QUE_START,
  .buf_end=CHAR_QUE_START+0x10000,
  .ip=CHAR_QUE_START,
  .op=CHAR_QUE_START
};

int rs323_buffered_out;

int rs232_test_in(void)
{
  return (*(volatile long*)UART1_SR2 & 0x1);
}

/* reads character throught CPU32Bug */
int rs232_getchar(void)
{ char ch;
  ch=0;
  while((*(volatile long*)UART1_SR2 & 0x1)==0);
  ch=(*(volatile long*)UART1_RXDATA & 0xff);  
  return ch;
}


int rs232_test_out(void)
{
  return (*(volatile long*)UART1_SR1 & 0x2000);
}

volatile int rs232_putchar(int ch)
{
  while((*(volatile long*)UART1_SR1 & 0x2000)==0);
  *(volatile long*)UART1_TXDATA=ch;  
  return ch;
}

int getchar(void)
{
  return rs232_getchar();
}

volatile int putchar(int ch)
{
  if(!rs323_buffered_out){
    ch=rs232_putchar(ch);
  }else{
    ch=char_que_put(&rs232_que_out,ch);
  }
  return ch;
}

void writestr(char *str)
{
  while(*str) putchar(*str++);
}

#define I2STR_PAD_0  0x100
#define I2STR_LONG   0x200
#define I2STR_NUMBER 0x400

int i2str(char *s,long val,int len,int form)
{
  int sig;
  int dig=1;
  int pad=0;
  unsigned base=form&0xff;
  unsigned long u;
  unsigned long mag;
  unsigned long num;
  if(!base) base=10;
  if((sig=(val<0)&&(base==10))) num=-val;
  else num=val;
 
  mag=1;
  u=base*mag;
  while(num>=u){
    dig++;
    mag=u;
    if(mag>(unsigned long)(~(unsigned long)0)/base) break;
    u*=base;
  }

  if(len){
    pad=len-dig;
    if(sig) pad--;
  }
  if(pad<0) pad=0;


  if(form&I2STR_PAD_0) {
    if(sig) *(s++)='-';
    while(pad){
      *(s++)='0';
      pad--;
    }
  }else{
    while(pad){
      *(s++)=' ';
      pad--;
    }
    if(sig) *(s++)='-';
  }

  while(dig--){
    u=num/mag;
    if(u>9) *(s++)='A'-10+u;
    else *(s++)='0'+u;
    num=num%mag;
    mag/=base;
  }
  *s=0;
  return 0;
}


void writelx(unsigned long l)
{
  char s[12];
  int i;
  
  for(i=0;i<10;i++)s[i]='0';
  s[i]=0;
  s[1]='x';

  i2str(s+2,l,8,I2STR_PAD_0 | 16);
  writestr(s);
}

int simple_printf(const char *f, ...)
{
  va_list ap;
  char c;
  int form;
  int len;
  long val;
  char s[12];
  va_start (ap, f);
  while((c=*(f++))!=0) {
    switch(c) {

      case '%':
        form=0; len=0;
        if(!*f) break;
        c=*(f++);
	if(c=='0') {
	  form|=I2STR_PAD_0;
          if(!*f) break;
          c=*(f++);
	}
        while((c>='0')&&(c<='9')){
	  len=len*10+c-'0';
          if(!*f) break;
          c=*(f++);
	}
	if(c=='l') {
	  form|=I2STR_LONG;
          if(!*f) break;
          c=*(f++);
	}
	switch(c) {
	  case 's':
	    writestr(va_arg(ap, char *));
	    break;
	  case 'p':
	    form|=I2STR_LONG;
	  case 'X':
	  case 'x':
	    form|=16;
	  case 'd':
	    form|=I2STR_NUMBER;
	    break;
	}
	if(form&I2STR_NUMBER) {
	  val=form&I2STR_LONG?va_arg(ap, long):va_arg(ap, int);
          i2str(s,val,len,form);
          writestr(s);
	}
	break;

      case '\n':
	putchar('\r');
        putchar('\n');
	break;

      default: putchar(c);
    }
  }
  va_end (ap);
  
  return 0;
}

int in_getnumber(long *pval, int base, int chars)
{
  char ch;
  char sgn=0;
  int cnt=0;
  long val=0;
  
  do{
    ch=getchar();
    if(!cnt&&(ch=='-')&&!sgn){
      sgn=1;
      continue;
    }
    if(ch<='9'){
      if(ch<'0'){
        if(!cnt)
	  return -1;
	else
	  break;
      }
      ch-='0';
      if(!base&&ch)
        base=10;
    }else{
      if((ch=='x') && !base && (cnt==1)){
        base=16;
	cnt=0;
	continue;
      }
      if(ch>='a') ch-='a'-'A';
      if(ch<'A'){
        if(!cnt)
	  return -1;
	else
	  break;
      }
      ch=ch-'A'+10;
      if(ch>=base){
        if(!cnt)
	  return -1;
	else
	  break;
      }
    }
    val*=base;
    val+=ch;
    cnt++;
  }while(--chars);
  *pval=sgn?-val:val;
  return cnt;
}

void _end_function(int code)
{ char *p;
  if(!code) p="\r\nProgram exited normaly\r\n";
  else p="\r\nProgram exited with error\r\n";
  writestr(p);
  while(1);
}

/***********************************************************/

void test_led1(void)
{
  *(volatile long*)PTD_GIUS=0x60000000; /* GIUS_D */
  *(volatile long*)PTD_OCR2=0x3C000000; /* OCR2_D */
  *(volatile long*)PTD_DDIR=0x60000000; /* DDIR_D */
  /*LED On:*/
  // *(volatile long*)PTD_DR=0x40000000; /* DR_D */
  /*LED Off:*/
  *(volatile long*)PTD_DR=0x20000000; /* DR_D */
}

long memory_test(long adr,long len)
{
 long a;
 long x,y;
 x=0;
 y=1;
 for(a=adr;a<adr+len;a+=4){
  *(long*)a=x;
  x+=0x01234567;
  y--;
  if(y==0){
   putchar('.');
   y=len/120;
  }
 } 
 x=0;
 y=1;
 for(a=adr;a<adr+len;a+=4){
  if(*(long*)a!=x){
   writestr("\r\n");
   return a;
  } 
  x+=0x01234567;
  y--;
  if(y==0){
   putchar('.');
   y=len/120;
  }
 } 
 writestr("\r\n");
 return 0;
}

/*void __main(void) {;};*/

/***********************************************************/

int usb_init_fl;
uint32_t usb_stat;
uint32_t usb_frame;
uint32_t usb_ep0_intr;
uint32_t usb_intr;

#ifdef WITH_USB
usb_device_t usb_mx1_device;
#endif /*WITH_USB*/

void usb_test(void)
{
  return;

  if(usb_stat != _reg_USBD_STAT){
    usb_stat=_reg_USBD_STAT;
    writestr("USB STAT ");
    writelx(usb_stat);
    writestr("\r\n");
  }
  /*if(usb_frame != _reg_USBD_FRAME){
    usb_frame =_reg_USBD_FRAME;
    writestr("USB FRAME ");
    writelx(usb_frame);
    writestr("\r\n");
  }*/
  if(usb_intr != _reg_USBD_INTR_STAT){
    usb_intr =_reg_USBD_INTR_STAT;
    if(usb_intr & ~MX1_USBD_INTR_SOF_m){
      writestr("USB INTR ");
      writelx(usb_intr);
      writestr("\r\n");
    }
  }
  if(usb_ep0_intr != _reg_USBD_EP(0).INTR_STAT){
    usb_ep0_intr = _reg_USBD_EP(0).INTR_STAT;
    writestr("EP0INTR ");
    writelx(usb_ep0_intr);
    writestr("\r\n");
  }

 #if !defined(WITH_USB) && !defined(WITH_USB_SIMPLE)
  if(usb_intr){
    if (usb_intr & MX1_USBD_INTR_SOF_m){
      _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m;    // signal CMD_OVER
    }
    _reg_USBD_INTR_STAT = usb_intr;
  }

  if(usb_ep0_intr){
    _reg_USBD_INTR_STAT = usb_ep0_intr;
  }
 #endif /*WITH_USB*/

}

/***********************************************************/

#ifdef WITH_IRQ_CATCH

unsigned long cpu_irq_stack[256];
unsigned long cpu_fiq_stack[256];
unsigned long cpu_expt_stack[256];

void do_irq(int excptnum, struct pt_regs *pt_regs)
{
  int i;
  static blink=0;
  if((++blink)&1)
    mx1_kbd_led_set(0x10);
  else
    mx1_kbd_led_set(0x00);

  //_reg_PTA_DDIR |= (1<<21); 
  //_reg_PTA_DR &= ~(1<<21);
  
  /*_reg_PTD_IMR &= ~(1<<20);*/
  _reg_PTD_ISR = (1<<20);
  
  writestr("EXCPT ");
  writelx(excptnum);
  for(i=0; i<18; i++){
    writestr(" ");
    writelx(pt_regs->uregs[i]);
  }
  writestr("\n\r");
}

void do_fiq(int excptnum, struct pt_regs *pt_regs)
{
}

void setup_irq_catch(void)
{
  int i;
  for(i=0;i<MAX_EXCEPTIONS;i++){
    set_cpu_exception_stack(i, (long)((char*)cpu_irq_stack+sizeof(cpu_expt_stack)-8));
  }
  set_cpu_exception_stack(ARM_EXCEPTION_IRQ, (long)((char*)cpu_irq_stack+sizeof(cpu_irq_stack)-8));
  set_cpu_exception_stack(ARM_EXCEPTION_FIQ, (long)((char*)cpu_irq_stack+sizeof(cpu_fiq_stack)-8));
  
  set_cpu_exception_handler(ARM_EXCEPTION_IRQ, do_irq);
  set_cpu_exception_handler(ARM_EXCEPTION_FIQ, do_fiq);
}

void mx1_test_irq(void)
{
  mx1_setup_irq_on_power_on();
  sti();
 #if 0
  asm (
    "	mov 	r0,#0x100\n\t"
    "	mov 	r1,#0x11\n\t"
    "	mov 	r2,#0x22\n\t"
    "	mov 	r3,#0x33\n\t"
    "	mov 	r4,#0x44\n\t"
    "	mov 	r5,#0x55\n\t"
    "	mov 	r6,#0x66\n\t"
    "	mov 	r7,#0x77\n\t"
    "	mov 	r8,#0x88\n\t"
    "	mov 	r9,#0x99\n\t"
    "	mov 	r10,#0xaa\n\t"
    "	mov 	r11,#0xbb\n\t"
    "	mov 	r12,#0xcc\n\t"
    "	mov 	r13,#0xdd\n\t"
    "	mov 	r14,#0xee\n"
    "	.word 	0xe7ffdeff\n"
    "	mov 	r0,#0x0a\n\t"
    "	mov 	r1,#0x1a\n\t"
    "	mov 	r2,#0x2a\n\t"
    "	.word 	0xe7ffdeff\n"
    "1:	b	1b\n\t"
  );
 #endif
}

#endif /*WITH_IRQ_CATCH*/

/***********************************************************/

typedef int (local_call_t)(void);

int main()
{ int x,s,hm;
  char c;
  int cont=0x18;
  int boot_select;

  mx1_hw_init();
 
  mx1_lcd_init();
  
  for(x=0;x<240;x++) mx1_lcd_base_addr[(320/2)*x+x]=0xff;
  
 #ifdef WITH_IRQ_CATCH
  setup_irq_catch();
 #endif /*WITH_IRQ_CATCH*/


  //*(long*)PTD_DR=0x20000200; /* DR_D */
  //*(long*)PTD_DR=0x20080200; /* DR_D */
  mx1_lcd_backlight_set(6);

  mx1_uart2_init();
  *(volatile long*)UART2_TXDATA='U';
  *(volatile long*)UART2_TXDATA='A';
  mx1_uart1_init();

  mx1_ioports_init();

  boot_select=mx1_get_boot_select();
  
  if((((long)&main)^0x10000000)&0xf0000000){
    /* use manual boot procedure if not running from FLASH */
    boot_select = 1;
  }
  
  usb_init_fl=0;

  writestr("MX_CPU1 v.1 ");
  writelx(*(int*)0x0021B804);
  writestr("\r\n");
  //printf("Printf test %d\n",12345678);
  simple_printf("Main is at %08lx, bs %d\n",(long)&main,boot_select);

 #ifdef WITH_GDB_STUB
  kgdb_serial_set_parameters(1, 19200);
  kgdb_init();
  writestr("kgdb_init done\r\n");
 #endif /*WITH_GDB_STUB*/
  
  //for(x=0;x<320/2*240;x++){
  //  mx1_lcd_base_addr[x]=0xff;
  //}
  
  if(!boot_select){
    /* start user application from FLASH */
    rs323_buffered_out=0;
    start_user_tag_application(&__flashusr_base, 0, simple_printf);
  }
  
 #ifdef WITH_USB
  usb_mx1_device.id = 1;
  usb_mx1_device.cntep = 0;

  #ifndef USB_MX1_DIRECT_FNC
  usb_mx1_device.init = usb_mx1_init;
  #endif /*USB_MX1_DIRECT_FNC*/
  //usb_debug_set_level(4);
  usb_debug_set_level(4);
  if(0) {
    usb_init(&usb_mx1_device);
    usb_connect(&usb_mx1_device);
    usb_init_fl=1;
  }
// cmd_usb_processor_init();
// usb_flags.bits.terminal_mode=1;
// usb_mx1_device.vendor_fnc=usb_msc1210_loader;
 #elif defined(WITH_USB_SIMPLE)
  usb_debug_set_level(4);
  usb_mx1_simple_init();
  writestr("\r\n");
  usb_init_fl=1;
 #endif /*WITH_USB*/
  
  rs323_buffered_out=1;
//  mx1_kbd_led_set(0x1);
//  mx1_sleep();
//  mx1_kbd_led_set(0x0);


  while(1){
 //   sl_udelay(1000);

    while(!rs232_test_in()) {
      if(usb_init_fl){
	usb_test();
       #ifdef WITH_USB
	if(usb_check_events( &usb_mx1_device)) {
          usb_control_response( &usb_mx1_device);
	}
       #elif defined(WITH_USB_SIMPLE)
        usb_mx1_simple_run();
       #endif /*WITH_USB_SIMPLE*/
      }
      if(rs323_buffered_out&&rs232_test_out()){
	int ch;
	if((ch=char_que_get(&rs232_que_out))>=0){
          rs232_putchar(ch);
	}
      }
    }
    
    c=getchar();
    if(c==0xa || c==0xd)   writestr("\r\n");
    else if(c=='f'){
      *(volatile short*)0x10000000=0x0090;
      writestr("FLASH ID=");
      writelx(*(volatile long*)0x10000000);
      writestr("\r\n");
      *(volatile short*)0x10000000=0x00FF;
    }
    else if(c=='R'){
      rs323_buffered_out=0;
    }
    else if(c=='r'){
      rs323_buffered_out=0;
      x=memory_test(0x08000000,0x02000000);
      if(x==0) writestr("SDRAM TEST OK\r\n");
      else {
        writestr("SDRAM TEST ERROR at a=");
        writelx(x);
        writestr("\r\n");
      }
      rs323_buffered_out=1;
    }    
 #ifdef WITH_SLEEP_WAKE 
    else if(c=='s'){
      rs323_buffered_out=0;
      writestr("Going to sleep\r\n");
      mx1_lcd_buff_enable(0);
      mx1_kbd_led_switch_off();
      mx1_sleep();
      mx1_lcd_buff_enable(1);
      mx1_kbd_led_shadow_wr();
      writestr("Waked up\r\n");
    }
 #endif /*WITH_SLEEP_WAKE*/
 #ifdef WITH_IRQ_CATCH
    else if(c=='i'){
      rs323_buffered_out=0;
      mx1_test_irq();
    }
 #endif /*WITH_IRQ_CATCH*/
 #ifdef WITH_GDB_STUB
    else if(c=='B'){
      rs323_buffered_out=0;
      writestr("BREAKPOINT");
      breakpoint();
    }
 #endif /*WITH_GDB_STUB*/
    else if(c=='t'){
      writestr("RTC=");
      s=*(volatile long*)RTC_SECOND;
      hm=*(volatile long*)RTC_HOURMIN;
      if(s!=(*(volatile long*)RTC_SECOND)){
       s=*(volatile long*)RTC_SECOND;
       hm=*(volatile long*)RTC_HOURMIN;
      }
      putchar((hm/256)/10+'0');
      putchar((hm/256)%10+'0');
      putchar(':');
      putchar((hm & 0x3f)/10+'0');
      putchar((hm & 0x3f)%10+'0');
      putchar(':');
      putchar((s & 0x3f)/10+'0');
      putchar((s & 0x3f)%10+'0');
      writestr("\r\n");
    }
    else if(c=='x'){
     break;
    }
    else if(c=='g'){
     unsigned int go_delay=0x10000;
     long go_addr;
     if(in_getnumber(&go_addr, 0, 0)>=0){
       rs323_buffered_out=0;
       simple_printf("\nGoTo 0x%08lx\n",go_addr);

       while(go_delay--){
	 _reg_USBD_INTR_MASK &= ~(MX1_USBD_INTR_SOF_m);
       }
      
       ((local_call_t*)go_addr)();
     }
     break;
    }
    else if(c>='0' && c<='7'){
     mx1_lcd_backlight_set(c-'0');
    }
    else if(c=='>'){
     cont++;
     if (cont>255) cont=255;
     mx1_lcd_contrast_set(cont);
     writelx(cont);
     writestr("\r\n");
    }
    else if(c=='<'){
     cont--;
     if (cont<0) cont=0;
     mx1_lcd_contrast_set(cont);
     writelx(cont);
     writestr("\r\n");
    }
    else if(c=='u'){
     #ifdef WITH_USB
      usb_init(&usb_mx1_device);
      usb_connect(&usb_mx1_device);
      usb_init_fl=1;
     #elif defined(WITH_USB_SIMPLE)
      usb_debug_set_level(4);
      usb_mx1_simple_init();
      writestr("\r\n");
     #else /* WITH_USB*/
      x=mx_usb_init_ll();
      writelx(x);
      writestr("\r\n");
     #endif /* WITH_USB*/
      usb_init_fl=1;
    }
    else if(c=='l'){
      char cmdline[512];
      char ch;
      int cmdlen=0;

      do{
	ch=getchar();
	if((ch==0xd)||(ch==0xa))
	  break;
	cmdline[cmdlen++]=ch;
      }while(1);

      cmdline[cmdlen]=0;

      rs323_buffered_out=0;
      simple_printf("\nStarting kernel\n");

      _reg_USBD_CTRL = 0;
      start_linux(MACH_TYPE_PIMX1, cmdline);
      simple_printf("Kernel not started\n");
    }
    else if(c=='e'){
      if(getchar()=='r')
      if(getchar()=='a')
      if(getchar()=='p')
      if(getchar()=='p'){
        int i, res;
	unsigned long addr;
        rs323_buffered_out=0;
        simple_printf("Application FLASH erase ");
	for(i=2;i<32;i++){
	  addr=0x10000000+0x20000*i;
	  
	  res=usb_mx1_simple_mass_erase(0, addr);
	  if(res<0)
	    simple_printf("Erase %08lx returned 0x%X\n", addr, res);
	  else
	    simple_printf(".");
        }
	*(unsigned long*)addr=0xffffffff;
	*(unsigned short*)addr=0xffff;
        rs323_buffered_out=1;
        simple_printf("\n");
      }
    }
    else if(c=='E'){
      if(getchar()=='r')
      if(getchar()=='a')
      if(getchar()=='s')
      if(getchar()=='e'){
        int res;
        res=usb_mx1_simple_mass_erase(0,0);
	simple_printf("Erase returned 0x%X\n", res);
      }
    }
    else if(c=='a'){ 
      _reg_USBD_CTRL = 0;
      rs323_buffered_out=0;
      start_user_tag_application(&__flashusr_base, 0, simple_printf);
    }
    else if(c=='p'){ 
      switch(getchar()){
        case 'u': 
	  simple_printf("USB registers:\n");
	  simple_printf(" STAT %08x  CTRL %08x ISTAT %08x  ENAB %08x\n",
	                _reg_USBD_STAT, _reg_USBD_CTRL, _reg_USBD_INTR_STAT, _reg_USBD_ENABLE);
	  simple_printf("STAT0 %08x ISTAT %08x FSTAT %08x FCTRL %08x\n",
	                _reg_USBD_EP(0).STAT, _reg_USBD_EP(0).INTR_STAT, 
			_reg_USBD_EP(0).FSTAT, _reg_USBD_EP(0).FCTRL);
	  break;
        case 'U':
	  simple_printf("USB data:\n");
	  while(!(_reg_USBD_EP(0).FSTAT&MX1_EP_FSTAT_EMPTY_m)){
	    simple_printf(" %02x",(((uint8_t*)&(_reg_USBD_EP(0).FDAT))[0]));
	  }
	  simple_printf("\n");
	  break;
      }
    }
    else if(c=='m'){
      mx1_hw_mclk_on();
    }
    else if(c==' '){ 
      getchar();
    } else putchar(c);
  } 

  mx1_lcd_backlight_set(0);
  writestr("EXIT\r\n");
  sl_udelay(10000);

//  while(1){
//  };
  
  return 0;
};

